@@ -309,7 +309,7 @@ ext2fs_copy_inode(EXT2FS_INFO *ext2fs, FS_INODE *fs_inode)
 
                 memcpy (ptr, fs_buf->data, read_count);
                 count += read_count;
-                ptr = (char *)(int)ptr + count;
+                ptr = (char *)((uintptr_t)ptr + count);
              }
 
 			 /* terminate the string */
@@ -906,7 +906,7 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 			fprintf(hFile, "Filetype, ");
 		if (getu32(fs, sb->s_feature_incompat) & 
 		  EXT2FS_FEATURE_INCOMPAT_RECOVER)
-			fprintf(hFile, "Recover, ");
+			fprintf(hFile, "Needs Recovery, ");
 		if (getu32(fs, sb->s_feature_incompat) & 
 		  EXT2FS_FEATURE_INCOMPAT_JOURNAL_DEV)
 			fprintf(hFile, "Journal Dev");
@@ -961,8 +961,6 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 	  getu32(fs, sb->s_free_inode_count));
 
 
-	/* @@@ NEED TO TEST THIS */
-#if 0
 	if (getu32(fs, sb->s_last_orphan)) {
 		u_int32_t	or_in;
 		fprintf(hFile, "Orphan Inodes: ");
@@ -982,8 +980,8 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 
 			or_in = fsi->dtime;
 		}
+		fprintf(hFile, "\n");
 	}
-#endif
 
 	fprintf(hFile, "\nCONTENT INFORMATION\n");
 	fprintf(hFile, "--------------------------------------------\n");
@@ -994,7 +992,7 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "Block Size: %lu\n", (ULONG)fs->block_size);
 
 	if (getu32(fs, sb->s_first_data_block)) 
-		fprintf(hFile, "Reserved Area Before Block Groups: %"PRIu32"\n",
+		fprintf(hFile, "Reserved Blocks Before Block Groups: %"PRIu32"\n",
 		  getu32(fs, sb->s_first_data_block));
 
 	fprintf(hFile, "Free Blocks: %"PRIu32"\n",
@@ -1045,9 +1043,13 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
 		/* only print the super block data if we are not in a sparse
 		 * group 
 		 */
-		if ((getu32(fs, ext2fs->fs->s_feature_ro_compat) & 
+		if ( ( (getu32(fs, ext2fs->fs->s_feature_ro_compat) & 
 	  	  EXT2FS_FEATURE_RO_COMPAT_SPARSE_SUPER) && 
-		  (cg_base != getu32(fs, ext2fs->group->bg_block_bitmap)) ) {
+		  (cg_base != getu32(fs, ext2fs->group->bg_block_bitmap)) ) || 
+
+		 ((getu32(fs, ext2fs->fs->s_feature_ro_compat) & 
+	  	  EXT2FS_FEATURE_RO_COMPAT_SPARSE_SUPER) == 0) ) {
+			       
 			OFF_T boff;
 
 			/* the super block is the first 1024 bytes */
@@ -1215,6 +1217,8 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	  (fs_inode->flags & FS_FLAG_META_ALLOC)?"":"Not ");
 
 	fprintf(hFile, "Group: %lu\n", (ULONG)ext2fs->grpnum);
+	fprintf(hFile, "Generation Id: %"PRIu32"\n", 
+	  getu32(fs, ext2fs->dinode->i_generation));
 
 	if (fs_inode->link)
 		fprintf(hFile, "symbolic link to: %s\n", fs_inode->link);
@@ -1225,6 +1229,13 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	make_ls(fs_inode->mode, ls, 12);
 	fprintf(hFile, "mode: %s\n", ls);
 
+	/* Print the device ids */
+	if (((fs_inode->mode & FS_INODE_FMT) == FS_INODE_BLK) ||
+	  ((fs_inode->mode & FS_INODE_FMT) == FS_INODE_CHR)) {
+		fprintf(hFile, "Device Major: %"PRIu8"   Minor: %"PRIu8"\n",
+		  ext2fs->dinode->i_block[0][1], ext2fs->dinode->i_block[0][0]);
+	}
+
 	if (getu32(fs, ext2fs->dinode->i_flags)) {
 		fprintf(hFile, "Flags: ");
 		if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_SECDEL)
@@ -1293,9 +1304,9 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 		/* Cycle through each entry - at the top of the block */
 		for (ea_entry = (ext2fs_ea_entry *)&ea_head->entry;
 
-		  ((int)ea_entry < ((int)fs_buf->data + fs->block_size - sizeof(ext2fs_ea_entry))) ;
+		  ((uintptr_t)ea_entry < ((uintptr_t)fs_buf->data + fs->block_size - sizeof(ext2fs_ea_entry))) ;
 
-		  ea_entry = (ext2fs_ea_entry *) ((int)ea_entry + 
+		  ea_entry = (ext2fs_ea_entry *) ((uintptr_t)ea_entry + 
 		    EXT2_EA_LEN(ea_entry->nlen))) {
 
 			char name[256];
@@ -1372,11 +1383,11 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 				}
 
 				/* The first entry starts after the header */
-				acl_lo = (ext2fs_pos_acl_entry_lo *)((int)acl_head + sizeof(ext2fs_pos_acl_head));
+				acl_lo = (ext2fs_pos_acl_entry_lo *)((uintptr_t)acl_head + sizeof(ext2fs_pos_acl_head));
 
 
 				/* Cycle through the values */
-				while ((int)acl_lo < ((int)fs_buf->data + getu16(fs, ea_entry->val_off) + getu32(fs, ea_entry->val_size))) {
+				while ((uintptr_t)acl_lo < ((uintptr_t)fs_buf->data + getu16(fs, ea_entry->val_off) + getu32(fs, ea_entry->val_size))) {
 
 					char perm[64];
 					int len;
@@ -1387,12 +1398,12 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 					switch (getu16(fs, acl_lo->tag)) {
 						case EXT2_PACL_TAG_USERO:
 
-							fprintf(hFile, "  uid: %d: %s\n", fs_inode->uid, perm);
+							fprintf(hFile, "  uid: %d: %s\n", (int)fs_inode->uid, perm);
 							len = sizeof(ext2fs_pos_acl_entry_sh);
 							break;
 
 						case EXT2_PACL_TAG_GRPO:
-							fprintf(hFile, "  gid: %d: %s\n", fs_inode->gid, perm);
+							fprintf(hFile, "  gid: %d: %s\n", (int)fs_inode->gid, perm);
 							len = sizeof(ext2fs_pos_acl_entry_sh);
 							break;
 						case EXT2_PACL_TAG_OTHER:
@@ -1421,7 +1432,7 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 							len = sizeof(ext2fs_pos_acl_entry_sh);
 							break;
 					}
-					acl_lo = (ext2fs_pos_acl_entry_lo *)((int)acl_lo + len);
+					acl_lo = (ext2fs_pos_acl_entry_lo *)((uintptr_t)acl_lo + len);
 				}
 			}
 			else {
@@ -1643,6 +1654,13 @@ ext2fs_open(const char *name, unsigned char ftype)
 	fs->istat = ext2fs_istat;
 	fs->close = ext2fs_close;
 
+
+	/* Journal */
+	fs->journ_inum = getu32(fs, ext2fs->fs->s_journal_inum);
+	fs->jblk_walk = ext2fs_jblk_walk;
+	fs->jentry_walk = ext2fs_jentry_walk;
+	fs->jopen = ext2fs_jopen;
+
 	/* allocate buffers */
 
 	/* inode map */
