@@ -299,6 +299,86 @@ typedef struct {
 } ext2fs_pos_acl_entry_lo;
 
 
+
+
+/************** JOURNAL ******************/
+
+/* These values are always in big endian */
+
+#define EXT2_JMAGIC	0xC03b3998
+
+typedef struct {
+	u_int8_t	magic[4];
+	u_int8_t        entrytype[4];
+	u_int8_t        entryseq[4];		/* sequence of this entry */
+	u_int8_t        bsize[4];		/* size of block */
+
+	u_int8_t        num_blk[4];		/* num of blks in journal */
+	u_int8_t        first_blk[4];		/* bl where log starts */
+	u_int8_t        start_seq[4];		/* first commit ID in log */
+	u_int8_t        start_blk[4]; 		/* journ blk for 1st valid entry */
+
+	u_int8_t        errno[4]; 		/* signed error number */
+
+/* the rest are not valid for v1 sb */
+	u_int8_t        feature_compat[4];
+	u_int8_t        feature_incompat[4];
+	u_int8_t        feature_ro_incompat[4];
+	u_int8_t        uuid[16];
+	u_int8_t        num_fs[4];		/* num of fs sharing log */
+	u_int8_t        dynsuper[4];		/* fs block of sb copy */
+	u_int8_t        max_trans[4];		/* limit of blk per trans */
+	u_int8_t        max_trans_data[4];	/* limit of data blocks per */
+	u_int8_t        reserved[176];
+	u_int8_t	id_fs[16][48];		/* Ids of fs sharing log */
+} ext2fs_journ_sb;
+
+
+#define EXT2_J_ETYPE_DESC	1	/* descriptor block */
+#define EXT2_J_ETYPE_COM	2	/* commit */
+#define EXT2_J_ETYPE_SB1	3	/* super block v1 */
+#define EXT2_J_ETYPE_SB2	4	/* sb v2 */
+#define EXT2_J_ETYPE_REV	5	/* revoke */
+
+
+/* Header that is used for all structures */
+typedef struct {
+	u_int8_t        magic[4];
+	u_int8_t        entry_type[4];
+	u_int8_t        entry_seq[4];
+} ext2fs_journ_head;
+
+
+/* dentry flags */
+#define EXT2_J_DENTRY_ESC	1	/* The orig block starts with magic */
+#define EXT2_J_DENTRY_SAMEID	2	/* Entry is for same id, so do not skip 16 ahead */
+#define EXT2_J_DENTRY_DEL	4	/* not currently used in src */
+#define EXT2_J_DENTRY_LAST	8	/* Last tag */
+
+/* Entry in the descriptor table */
+typedef struct {
+	u_int8_t        fs_blk[4];
+	u_int8_t        flag[4];
+} ext2fs_journ_dentry;
+
+
+/* Journal Info */
+typedef struct {
+
+	FS_INODE	*fs_inode;
+	u_int32_t	j_inum;
+
+	u_int32_t	bsize;
+	u_int32_t	first_block;
+	u_int32_t	last_block;
+
+	u_int32_t	start_seq;
+	u_int32_t	start_blk;
+
+} EXT2FS_JINFO;
+#endif
+
+
  /*
   * Structure of an ext2fs file system handle.
   */
@@ -317,8 +397,12 @@ typedef struct {
 	int     groups_count;       /* nr of descriptor group blocks */
 	u_int8_t	deentry_type;	/* v1 or v2 of dentry */
 	u_int16_t	inode_size;	/* size of each inode */
+	EXT2FS_JINFO	*jinfo;
 } EXT2FS_INFO;
 
 extern void ext2fs_dent_walk(FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
+extern void ext2fs_jentry_walk (FS_INFO *, int, FS_JENTRY_WALK_FN, char *);
+extern void ext2fs_jblk_walk (FS_INFO *, u_int32_t, u_int32_t, int, FS_JBLK_WALK_FN, char *);
+extern void ext2fs_jopen (FS_INFO *, u_int32_t);
+
 
-#endif
