@@ -317,7 +317,7 @@ ext2fs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 {
     int len = (dirleft < size) ? dirleft : size;
     memcpy (curdirptr, buf, len); 
-    curdirptr = (char *) ((int)curdirptr + len);
+    curdirptr = (char *) ((uintptr_t)curdirptr + len);
     dirleft -= len;
     if (dirleft)
         return WALK_CONT;
@@ -351,17 +351,17 @@ ext2fs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 	fs_inode = fs->inode_lookup(fs, inode);
 	size = roundup (fs_inode->size, fs->block_size);
 
-    /* make a copy of the directory contents that we can process */
-    if (curdirptr != NULL)
-        error ("ext2fs_dent_walk: Curdirptr is set! recursive?"); 
+	/* make a copy of the directory contents that we can process */
+	if (curdirptr != NULL)
+		error ("ext2fs_dent_walk: Curdirptr is set! recursive?"); 
   
-    curdirptr = dirbuf = mymalloc(size); 
-    dirleft = size;
+	curdirptr = dirbuf = mymalloc(size); 
+	dirleft = size;
     
-    fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID,
+	fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID,
 	  ext2fs_dent_action, "");
 
-    curdirptr = NULL;
+	curdirptr = NULL;
 	dirptr = dirbuf;
 
 	while (size > 0) {
@@ -377,7 +377,7 @@ ext2fs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 		else
 			break;
 
-		dirptr = (char *)((int)dirptr + len);
+		dirptr = (char *)((uintptr_t)dirptr + len);
 	}
 
 	free (dirbuf);
