@@ -628,7 +628,7 @@ fatfs_make_root(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 
 	fs_inode->flags = (FS_FLAG_META_USED | FS_FLAG_META_ALLOC);
 
-    fs_inode->uid = fs_inode->gid = 0;
+	fs_inode->uid = fs_inode->gid = 0;
 	fs_inode->mtime = fs_inode->atime = fs_inode->ctime = fs_inode->dtime = 0;
 
 	if (fs_inode->name == NULL) {
@@ -650,10 +650,10 @@ fatfs_make_root(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 		fs_inode->direct_addr[0] = 1;
 
 		/* difference between end of FAT and start of clusters */
-    	snum = fatfs->firstclustsect - fatfs->firstdatasect;
+		snum = fatfs->firstclustsect - fatfs->firstdatasect;
 
 		/* number of bytes */
-    	fs_inode->size = snum * fatfs->ssize; 
+		fs_inode->size = snum * fatfs->ssize; 
 	}
 	else {
 		/* Get the number of allocated clusters */
@@ -668,7 +668,7 @@ fatfs_make_root(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 			clust = getFAT(fatfs, clust);
 		}
 
-    	fs_inode->size = cnum * fatfs->csize * fatfs->ssize; 
+		fs_inode->size = cnum * fatfs->csize * fatfs->ssize; 
 	}
 }
 
@@ -1331,19 +1331,23 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 		  sb->a.f32.fs_type[3], sb->a.f32.fs_type[4], sb->a.f32.fs_type[5],
 		  sb->a.f32.fs_type[6], sb->a.f32.fs_type[7]);
 
-		fs_read_block(fs, fat_fsinfo_buf, sizeof(fatfs_fsinfo), 
-		  getu16(fs, sb->a.f32.fsinfo), 
-		  "read fs_info");
 
-		fat_info = (fatfs_fsinfo *)fat_fsinfo_buf->data;
-		fprintf(hFile, "Next Free Sector (FS Info): %lu\n",
-		  (ULONG)FATFS_CLUST_2_SECT(fatfs, getu32(fs, fat_info->nextfree)));
+		/* Process the FS info */
+		if (getu16(fs, sb->a.f32.fsinfo)) {
+			fs_read_block(fs, fat_fsinfo_buf, sizeof(fatfs_fsinfo), 
+			  getu16(fs, sb->a.f32.fsinfo), 
+			  "read fs_info");
 
-		fprintf(hFile, "Free Sector Count (FS Info): %lu\n",
-		  (ULONG)
-		  (getu32(fs, fat_info->freecnt) * fatfs->csize));
+			fat_info = (fatfs_fsinfo *)fat_fsinfo_buf->data;
+			fprintf(hFile, "Next Free Sector (FS Info): %lu\n",
+			  (ULONG)FATFS_CLUST_2_SECT(fatfs, getu32(fs, fat_info->nextfree)));
 
-		fs_buf_free(fat_fsinfo_buf);
+			fprintf(hFile, "Free Sector Count (FS Info): %lu\n",
+			  (ULONG)
+			  (getu32(fs, fat_info->freecnt) * fatfs->csize));
+
+			fs_buf_free(fat_fsinfo_buf);
+		}
 	}
 
 	fs_buf_free(fs_buf);
@@ -1397,11 +1401,25 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 	}
 	else {
 		u_int32_t x = fatfs->csize * (fatfs->lastclust - 1);
+		u_int32_t clust, clust_p;
 
 		fprintf(hFile, "** Cluster Area: %lu - %lu\n",
 		  (ULONG)fatfs->firstclustsect, 
 		  (ULONG)(fatfs->firstclustsect + x - 1));
 
+
+		clust_p = fatfs->rootsect;
+		clust = FATFS_SECT_2_CLUST(fatfs, fatfs->rootsect);
+		while ((clust) && (0 == FATFS_ISEOF(clust, FATFS_32_MASK))) {
+			clust_p = clust;
+			clust = getFAT(fatfs, clust);
+		}
+		
+		fprintf(hFile, "*** Root Directory: %lu - %lu\n",
+		  (ULONG)fatfs->rootsect, 
+		  (ULONG)(FATFS_CLUST_2_SECT(fatfs, clust_p + 1) - 1));
+
+
 		if ((fatfs->firstclustsect + x - 1) != fs->last_block) {
 			fprintf(hFile, "** Non-clustered: %lu - %lu\n",
 			  (ULONG)(fatfs->firstclustsect + x),
@@ -1574,7 +1592,8 @@ fatfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	}
 
 	fprintf(hFile, "Size: %lu\n", (ULONG) fs_inode->size);
-	fprintf(hFile, "Num of links: %lu\n", (ULONG) fs_inode->nlink);
+	/* This value is fake in FAT, so there is no point in printing it here */
+	//fprintf(hFile, "Num of links: %lu\n", (ULONG) fs_inode->nlink);
 
 	if (fs_inode->name) {
 		fs_name = fs_inode->name;
@@ -1640,6 +1659,29 @@ fatfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 }
 
 
+void
+fatfs_jopen (FS_INFO *fs, u_int32_t inum)
+{
+        fprintf(stderr, "Error: FAT does not have a journal\n");
+        exit(1);
+}
+
+void
+fatfs_jentry_walk (FS_INFO *fs, int flags, FS_JENTRY_WALK_FN action, char *ptr)
+{
+        fprintf(stderr, "Error: FAT does not have a journal\n");
+        exit(1);
+}
+
+
+void
+fatfs_jblk_walk (FS_INFO *fs, u_int32_t start, u_int32_t end, int flags, FS_JBLK_WALK_FN action, char *ptr)
+{
+        fprintf(stderr, "Error: FAT does not have a journal\n");
+        exit(1);
+}
+
+
 /* fatfs_close - close an fatfs file system */
 static void
 fatfs_close(FS_INFO *fs)
@@ -1862,5 +1904,10 @@ fatfs_open(const char *name, unsigned char ftype)
 	fs->istat = fatfs_istat;
 	fs->close = fatfs_close;
 
+	fs->jblk_walk = fatfs_jblk_walk;
+	fs->jentry_walk = fatfs_jentry_walk;
+	fs->jopen = fatfs_jopen;
+	fs->journ_inum = 0;
+
 	return (fs);
 }
