@@ -558,7 +558,7 @@ fatfs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 
 	/* Copy the sector into a buffer and increment the pointers */
     memcpy (g_curdirptr, buf, len);
-    g_curdirptr = (char *) ((int)g_curdirptr + len);
+    g_curdirptr = (char *) ((uintptr_t)g_curdirptr + len);
     g_dirleft -= len;
 
 	/* fill in the stack of addresses of sectors 
@@ -598,7 +598,6 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 	FATFS_INFO	*fatfs = (FATFS_INFO *)fs;
 	char *dirbuf, *dirptr;
 	DADDR_T *addrbuf;
-	int addrmax;
 	int i;
 
 	if ((inode < fs->first_inum) || (inode > fs->last_inum))
@@ -627,7 +626,6 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 	 * the file_walk functionality 
 	 */
 
-
 	/* Allocate a buffer for the directory contents */
 	if (g_curdirptr != NULL)
 		error ("fatfs_dent_walk: g_curdirptr is set! recursive?");
@@ -685,9 +683,6 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 	}
 
 
-	/* How many sector addresses do we have? */
-	addrmax = g_addridx;
-
 	/* Reset the local pointer because we could have done a realloc */
 	addrbuf = g_curaddrbuf;
 
@@ -723,8 +718,8 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 			  (ULONG)len, (ULONG)retval);
 		}
 
-        dirptr = (char *)((int)dirptr + len);
-    }
+        dirptr = (char *)((uintptr_t)dirptr + len);
+	}
 
 	free (dirbuf);
 	free(addrbuf);
