@@ -63,7 +63,7 @@ static ffs_cgd
      * group always fits within a logical disk block (as promised in the
      * 4.4BSD <ufs/ffs/fs.h> include file).
      */
-	if (ffs->cg_buf == 0)
+	if (ffs->cg_buf == NULL)
 		ffs->cg_buf = fs_buf_alloc(ffs->ffsbsize_b);
 
 	addr = cgtod_lcl(fs, ffs->fs.sb1, cgnum);
@@ -80,7 +80,7 @@ static ffs_cgd
 static void 
 ffs_cgroup_free(FFS_INFO *ffs)
 {
-    if (ffs->cg_buf)
+	if (ffs->cg_buf)
 		fs_buf_free(ffs->cg_buf);
 }
 
@@ -92,35 +92,79 @@ ffs_cgroup_free(FFS_INFO *ffs)
 static void
 ffs_dinode_lookup(FFS_INFO *ffs, INUM_T inum)
 {
-    DADDR_T addr;
-    OFF_T     offs;
+	DADDR_T addr;
+	OFF_T     offs;
 	FS_INFO *fs = (FS_INFO *)&ffs->fs_info;
 
-    /*
-     * Sanity check.
-     */
-    if (inum < fs->first_inum || inum > fs->last_inum)
+	/*
+	 * Sanity check.
+	 */
+	if (inum < fs->first_inum || inum > fs->last_inum)
 		error("invalid inode number: %lu", (ULONG) inum);
 
-    /*
-     * Allocate/read the inode buffer on the fly.
+	/*
+	 * Allocate/read the inode buffer on the fly.
 	 */
 	if (ffs->dino_buf == 0)
 		ffs->dino_buf = fs_buf_alloc(ffs->ffsbsize_b);
 
-    addr = itod_lcl(fs, ffs->fs.sb1, inum);
-    if (ffs->dino_buf->addr != addr)
-		fs_read_block(fs, ffs->dino_buf, ffs->dino_buf->size, addr,
-		      "inode block");
 
-    /*
-     * Copy the inode, in order to avoid alignment problems when accessing
-     * structure members.
-     */
-    offs = itoo_lcl(fs, ffs->fs.sb1, inum) * sizeof(ffs_inode1);
+	/*
+	 * Copy the inode into the buffer
+	 */
 
-    memcpy((char *) ffs->dinode, ffs->dino_buf->data + offs, 
- 	  sizeof(ffs_inode1));
+
+	/* UFS2 is different because it does not initialize all inodes
+	 * when the file system is created.  Therefore we need to check
+	 * the group descriptor to find out if this is in the valid
+	 * range
+	 */
+	if (fs->ftype == FFS_2) {
+		ffs_cgd2 *cg2;
+		int cg_num = itog_lcl(fs, ffs->fs.sb1, inum);
+
+		/* Lookup the cylinder group descriptor if it isn't
+		 * cached
+		 */
+		if ((ffs->cg_buf == NULL) || (cg_num != ffs->cg_num)) {
+			cg2 = (ffs_cgd2 *)ffs_cgroup_lookup(ffs, cg_num);
+		} 
+		else {
+	       		cg2 = (ffs_cgd2 *)ffs->cg_buf->data;
+		}
+
+
+		/* If the inode is not init, then do not worry about it */
+		if ((inum - cg_num * getu32(fs, ffs->fs.sb2->cg_inode_num)) >=
+		  getu32(fs, cg2->cg_initediblk)) {
+			memset((char *) ffs->dinode, 0, sizeof(ffs_inode2));
+		}
+
+		else {
+			/* Get the base and offset addr for the inode in the tbl */
+			addr = itod_lcl(fs, ffs->fs.sb1, inum);
+
+			if (ffs->dino_buf->addr != addr)
+				fs_read_block(fs, ffs->dino_buf, ffs->dino_buf->size, addr,
+				      "inode block");
+
+			offs = itoo_lcl(fs, ffs->fs.sb2, inum) * sizeof(ffs_inode2);
+
+			memcpy((char *) ffs->dinode, ffs->dino_buf->data + offs, 
+			  sizeof(ffs_inode2));
+		}
+	} 
+	else {
+		addr = itod_lcl(fs, ffs->fs.sb1, inum);
+		if (ffs->dino_buf->addr != addr)
+			fs_read_block(fs, ffs->dino_buf, ffs->dino_buf->size, addr,
+			      "inode block");
+
+		offs = itoo_lcl(fs, ffs->fs.sb1, inum) * sizeof(ffs_inode1);
+
+		memcpy((char *) ffs->dinode, ffs->dino_buf->data + offs, 
+		  sizeof(ffs_inode1));
+	}
 
 	ffs->inum = inum;
 
@@ -245,7 +289,7 @@ ffs_copy_inode(FFS_INFO *ffs, FS_INODE *fs_inode)
 
 					memcpy (ptr, fs_buf->data, read_count);
 					count += read_count;
-					ptr = (char *)(int)ptr + read_count;
+					ptr = (char *)((uintptr_t)ptr + read_count);
 				 }
 				 /* terminate the string */
 				 *ptr = '\0';
@@ -254,9 +298,9 @@ ffs_copy_inode(FFS_INFO *ffs, FS_INODE *fs_inode)
 			}
 		} /* end of symlink */
 	}
-	/* FFS_2 */
-	else {
-		ffs_inode2	*in = (ffs_inode2 *)ffs->dinode;
+	/* FFS_1B - Solaris */
+	else if (fs->ftype == FFS_1B) {
+		ffs_inode1b	*in = (ffs_inode1b *)ffs->dinode;
 
 		fs_inode->mode = getu16(fs, in->di_mode);
 		fs_inode->nlink = gets16(fs, in->di_nlink);
@@ -302,7 +346,7 @@ ffs_copy_inode(FFS_INFO *ffs, FS_INODE *fs_inode)
 
 				memcpy (ptr, fs_buf->data, read_count);
 				count += read_count;
-				ptr = (char *)(int)ptr + read_count;
+				ptr = (char *)((uintptr_t)ptr + read_count);
 			 }
 
 			/* terminate the string */
@@ -311,6 +355,101 @@ ffs_copy_inode(FFS_INFO *ffs, FS_INODE *fs_inode)
 			 fs_buf_free(fs_buf);
 		 }
 	}
+	else if (fs->ftype == FFS_2) {
+		ffs_inode2	*in = (ffs_inode2 *)ffs->dinode;
+
+		fs_inode->mode = getu16(fs, in->di_mode);
+		fs_inode->nlink = gets16(fs, in->di_nlink);
+		fs_inode->size = getu64(fs, in->di_size);
+		fs_inode->uid = getu32(fs, in->di_uid);
+		fs_inode->gid = getu32(fs, in->di_gid);
+
+		fs_inode->mtime = (time_t)gets64(fs, in->di_mtime);
+		fs_inode->atime = (time_t)gets64(fs, in->di_atime);
+		fs_inode->ctime = (time_t)gets64(fs, in->di_ctime);
+
+		if (fs_inode->direct_count != FFS_NDADDR || 
+		  fs_inode->indir_count != FFS_NIADDR)
+			fs_inode_realloc(fs_inode, FFS_NDADDR, FFS_NIADDR);
+
+		for (i = 0; i < FFS_NDADDR; i++)
+			fs_inode->direct_addr[i] = gets64(fs, in->di_db[i]);
+
+		for (i = 0; i < FFS_NIADDR; i++)
+			fs_inode->indir_addr[i] = gets64(fs, in->di_ib[i]);
+
+
+		/* set the link string (if the file is a link) 
+		 * The size check is a sanity check so that we don't try and allocate
+		 * a huge amount of memory for a bad inode value
+		 */
+		if (((fs_inode->mode & FS_INODE_FMT) == FS_INODE_LNK) && 
+		  (fs_inode->size < FFS_MAXPATHLEN) && (fs_inode->size >= 0)) {
+
+			fs_inode->link = mymalloc(fs_inode->size+1);
+
+			count = 0;	/* index into the link array */
+
+			/* it is located directly in the pointers  
+			 * Only the new style inode has this "fast link"
+			 */
+			if (fs_inode->size < 8 * (FFS_NDADDR + FFS_NIADDR)) {
+				char *ptr;
+
+				/* Direct block pointer locations */
+				for (i = 0; i < FFS_NDADDR && count < fs_inode->size; i++) {
+					ptr = (char *)&in->di_db[i];
+					for (j = 0; j < 8 && count < fs_inode->size; j++) 
+						fs_inode->link[count++] = ptr[j];
+				}
+
+				/* indirect block pointers */
+				for (i = 0; i < FFS_NIADDR && count < fs_inode->size; i++) {
+					ptr = (char *)&in->di_ib[i];
+					for (j = 0; j < 8 && count < fs_inode->size; j++) 
+						fs_inode->link[count++] = ptr[j];
+				}
+
+				fs_inode->link[count] = '\0';
+
+				/* clear the values to avoid other code from reading them */
+				for (i = 0; i < FFS_NDADDR; i++)
+					fs_inode->direct_addr[i] = 0;
+
+				for (i = 0; i < FFS_NIADDR; i++)
+					fs_inode->indir_addr[i] = 0;
+			}
+
+			/* it is in blocks (the regular way) */
+			else {
+				FS_BUF *fs_buf = fs_buf_alloc(fs->block_size);
+				char *ptr = fs_inode->link;
+
+				/* there is a max link length of 1000, so we should never
+				 * need the indirect blocks
+				 */
+				for (i = 0; i < FFS_NDADDR && count < fs_inode->size; i++) {
+					/* Do we need the entire block, or just part of it? */
+					int read_count = (fs_inode->size - count < fs->block_size)?
+						fs_inode->size - count : fs->block_size;
+
+					fs_read_block(fs, fs_buf, fs->block_size, 
+					  fs_inode->direct_addr[i], "link block");
+
+					memcpy (ptr, fs_buf->data, read_count);
+					count += read_count;
+					ptr = (char *)((uintptr_t)ptr + read_count);
+				 }
+				 /* terminate the string */
+				 *ptr = '\0';
+
+				 fs_buf_free(fs_buf);
+			}
+		} /* end of symlink */
+	}
+	else {
+		error ("copy_inode: Unknown FFS Type");
+	}
 
 	/* set the flags */
 	cg_num = itog_lcl(fs, ffs->fs.sb1, ffs->inum);
@@ -380,14 +519,13 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
     FS_INODE *fs_inode = fs_inode_alloc(FFS_NDADDR, FFS_NIADDR);
     int     myflags;
     INUM_T  ibase = 0;
-	ffs_inode1	*in;
 
     /*
      * Sanity checks.
      */
-    if (start < fs->first_inum || start > fs->last_inum)
+	if (start < fs->first_inum || start > fs->last_inum)
 		error("%s: invalid start inode number: %lu", myname, (ULONG) start);
-    if (last < fs->first_inum || last > fs->last_inum || last < start)
+	if (last < fs->first_inum || last > fs->last_inum || last < start)
 		error("%s: invalid last inode number: %lu", myname, (ULONG) last);
 
     /*
@@ -401,7 +539,7 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 	 	*/
 		cg_num = itog_lcl(fs, ffs->fs.sb1, inum);
 
-		if (cg == 0 || gets32(fs, cg->cg_cgx) != cg_num) {
+		if (cg_num != ffs->cg_num) {
 			cg = ffs_cgroup_lookup(ffs, cg_num);
 			inosused = (unsigned char *) cg_inosused_lcl(fs, cg);
 			ibase = cg_num * gets32(fs, ffs->fs.sb1->cg_inode_num);
@@ -417,24 +555,47 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 		ffs_dinode_lookup(ffs, inum);
 
-		/* both inode forms are the same for the required fields */
-		in = (ffs_inode1 *)ffs->dinode;
 
-		/*
-		 * Apply the linked/unlinked restriction.
-		 */
-		myflags |= (gets16(fs, in->di_nlink) ? 
-		  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
-		if ((flags & myflags) != myflags)
-			continue;
+		if ((fs->ftype == FFS_1) ||
+		  (fs->ftype == FFS_1B)) {
+			/* both inode forms are the same for the required fields */
+			ffs_inode1 *in1 = (ffs_inode1 *)ffs->dinode;
 
-		/*
-		 * Apply the used/unused restriction.
-		 */
-		myflags |= (gets32(fs, in->di_ctime) ? 
-		  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
-		if ((flags & myflags) != myflags)
-			continue;
+			/*
+			 * Apply the linked/unlinked restriction.
+			 */
+			myflags |= (gets16(fs, in1->di_nlink) ? 
+			  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
+			if ((flags & myflags) != myflags)
+				continue;
+
+			/*
+			 * Apply the used/unused restriction.
+			 */
+			myflags |= (gets32(fs, in1->di_ctime) ? 
+			  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
+			if ((flags & myflags) != myflags)
+				continue;
+		}
+		else {
+			ffs_inode2 *in2 = (ffs_inode2 *)ffs->dinode;
+
+			/*
+			 * Apply the linked/unlinked restriction.
+			 */
+			myflags |= (gets16(fs, in2->di_nlink) ? 
+			  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
+			if ((flags & myflags) != myflags)
+				continue;
+
+			/*
+			 * Apply the used/unused restriction.
+			 */
+			myflags |= (gets64(fs, in2->di_ctime) ? 
+			  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
+			if ((flags & myflags) != myflags)
+				continue;
+		}
 
 		/*
 		 * Fill in a file system-independent inode structure and pass control
@@ -534,7 +695,6 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 		   (unsigned long) cg_num, (unsigned long) addr,
 		   (unsigned long) dbase);
 
-
 	/*
 	 * Prepare for file systems that have a partial last logical block.
 	 */
@@ -616,25 +776,25 @@ static int
 ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
   size_t length, DADDR_T addr, int flags, FS_FILE_WALK_FN action, char *ptr)
 {
-    int     read_count;
-    int     myflags;
+	int     read_count;
+	int     myflags;
 
 	read_count = (length < buf->size ? length : buf->size);
 
-    if (addr > fs->last_block) {
-        if (flags & FS_FLAG_FILE_NOABORT)  {
+	if (addr > fs->last_block) {
+		if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
 				  "Invalid direct block address (too large): %lu",
 				  (ULONG)addr);
 			}
-            return 0;
+			return 0;
 		}
 		else {
 			error ("Invalid direct block address (too large): %lu",
 			  (ULONG)addr);
 		}
-    }    
+	}    
 
 	/* Check if this goes over the end of the image 
 	 * This exists when the image size is not a multiple of the block
@@ -645,18 +805,19 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
 		read_count = (fs->last_block - addr + 1) * fs->block_size;
 	}
 
-    if (addr == 0) {
+	if (addr == 0) {
 		if (0 == (flags & FS_FLAG_FILE_NOSPARSE)) {
 			myflags = FS_FLAG_DATA_CONT;
 
-            if ((flags & FS_FLAG_FILE_AONLY) == 0)
-                memset(buf->data, 0, read_count);
+			if ((flags & FS_FLAG_FILE_AONLY) == 0)
+				memset(buf->data, 0, read_count);
 
-            if (WALK_STOP == action(fs, addr, buf->data, read_count, 
+			if (WALK_STOP == action(fs, addr, buf->data, read_count, 
 			   myflags, ptr)) 
 				return 0;
-        }
-    } else {
+		}
+	} 
+	else {
 		myflags = FS_FLAG_DATA_CONT;
 
 		if ((flags & FS_FLAG_FILE_AONLY) == 0)
@@ -665,8 +826,8 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
 
 		if (WALK_STOP == action(fs, addr, buf->data, read_count, myflags, ptr))
 			return 0;
-    }
-    return (read_count);
+	}
+	return (read_count);
 }
 
 
@@ -680,79 +841,123 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 {   
     char   *myname = "ffs_file_walk_indir";
     size_t   todo_count = length;
-    DADDR_T *iaddr;
     int     n;
 
     if (verbose)
         fprintf(logfp, "%s: level %d block %lu\n", myname, level, (ULONG) addr)
 ;   
     
-    if (addr > fs->last_block) {
-        if (flags & FS_FLAG_FILE_NOABORT)  {
+	if (addr > fs->last_block) {
+		if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
         		  "Invalid indirect block address (too large): %lu",
 				  (ULONG)addr);
 			}
-            return 0;
+			return 0;
 		}
 		else  {
-        	error ("Invalid indirect block address (too large): %lu",
+			error ("Invalid indirect block address (too large): %lu",
 			  (ULONG)addr);
 		}
-    }    
+	}    
 
     /*
      * Read a block of disk addresses.
      */ 
-    if (addr == 0)
-        memset(buf[level]->data, 0, buf[level]->size); 
-    else
-        fs_read_block(fs, buf[level], buf[level]->size, addr,
-              "disk address block");
+	if (addr == 0)
+		memset(buf[level]->data, 0, buf[level]->size); 
+	else
+		fs_read_block(fs, buf[level], buf[level]->size, addr,
+		  "disk address block");
+
 
 	/* we only call the action  if the META flag is set */
-    if (flags & FS_FLAG_FILE_META) {
-        int myflags = FS_FLAG_DATA_META;
-        action(fs, addr, buf[level]->data, buf[level]->size, myflags, ptr);
-    }
+	if (flags & FS_FLAG_FILE_META) {
+		int myflags = FS_FLAG_DATA_META;
+		action(fs, addr, buf[level]->data, buf[level]->size, myflags, ptr);
+	}
+
 
     /*   
      * For each disk address, copy a direct block or process an indirect
      * block.
      */     
-    iaddr = (DADDR_T *) buf[level]->data; 
-    for (n = 0; todo_count > 0 && n < buf[level]->size / sizeof(*iaddr); n++) {
-		int prevcnt = todo_count;
+	if ((fs->ftype == FFS_1) ||
+	  (fs->ftype == FFS_1B)) {
 
-		if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
-			if (flags & FS_FLAG_FILE_NOABORT) {
-				if (verbose) {
-					fprintf (logfp, 
-					  "Invalid address in indirect list (too large): %lu",
+		DADDR_T *iaddr = (DADDR_T *) buf[level]->data; 
+		for (n = 0; 
+		  todo_count > 0 && n < buf[level]->size / sizeof(*iaddr); 
+		  n++) {
+
+			int prevcnt = todo_count;
+
+			if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
+				if (flags & FS_FLAG_FILE_NOABORT) {
+					if (verbose) {
+						fprintf (logfp, 
+						  "Invalid address in indirect list (too large): %lu",
+						  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]));
+					}
+					return 0;
+				}
+				else {
+					error ("Invalid address in indirect list (too large): %lu",
 					  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]));
 				}
-				return 0;
-			}
-			else {
-				error ("Invalid address in indirect list (too large): %lu",
-                  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]));
-			}
-        } 
+			} 
 
-        if (level == 1)
-            todo_count -= ffs_file_walk_direct(fs, buf[0], todo_count,
-              getu32(fs, (u_int8_t *)&iaddr[n]), flags, action, ptr);
-        else
-            todo_count -= ffs_file_walk_indir(fs, buf, todo_count,
-              getu32(fs, (u_int8_t *)&iaddr[n]), level - 1, flags, action, ptr)
+			if (level == 1)
+				todo_count -= ffs_file_walk_direct(fs, buf[0], todo_count,
+				  getu32(fs, (u_int8_t *)&iaddr[n]), flags, action, ptr);
+			else
+				todo_count -= ffs_file_walk_indir(fs, buf, todo_count,
+				  getu32(fs, (u_int8_t *)&iaddr[n]), level - 1, flags, action, ptr)
 ;
-		/* This occurs when 0 is returned, which means we want to exit */
-		if (prevcnt == todo_count)
-			return 0;
-    }
+			/* This occurs when 0 is returned, which means we want to exit */
+			if (prevcnt == todo_count)
+				return 0;
+		}
+	}
+	else {
+		u_int64_t *iaddr = (u_int64_t *) buf[level]->data; 
+		for (n = 0; 
+		  todo_count > 0 && n < buf[level]->size / sizeof(*iaddr); 
+		  n++) {
 
-    return (length - todo_count);
+			int prevcnt = todo_count;
+
+			if (getu64(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
+				if (flags & FS_FLAG_FILE_NOABORT) {
+					if (verbose) {
+						fprintf (logfp, 
+						  "Invalid address in indirect list (too large): %lu",
+						  (ULONG)getu64(fs, (u_int8_t *)&iaddr[n]));
+					}
+					return 0;
+				}
+				else {
+					error ("Invalid address in indirect list (too large): %lu",
+					  (ULONG)getu64(fs, (u_int8_t *)&iaddr[n]));
+				}
+			} 
+
+			/* @@@ Now we could be reducing a 64-bit adress to 32-bit - this will be fixed with v2 */
+			if (level == 1)
+				todo_count -= ffs_file_walk_direct(fs, buf[0], todo_count,
+				  (DADDR_T)getu64(fs, (u_int8_t *)&iaddr[n]), flags, action, ptr);
+			else
+				todo_count -= ffs_file_walk_indir(fs, buf, todo_count,
+				  (DADDR_T)getu64(fs, (u_int8_t *)&iaddr[n]), level - 1, flags, action, ptr)
+;
+			/* This occurs when 0 is returned, which means we want to exit */
+			if (prevcnt == todo_count)
+				return 0;
+		}
+	}
+
+	return (length - todo_count);
 }
 
 /*  
@@ -783,19 +988,19 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
      * for files. The level 1.. buffers are sized to the block size used for
      * indirect blocks.
      */
-    buf = (FS_BUF **) mymalloc(sizeof(*buf) * (inode->indir_count + 1));
-    buf[0] = fs_buf_alloc(ffs->ffsbsize_b);
+	buf = (FS_BUF **) mymalloc(sizeof(*buf) * (inode->indir_count + 1));
+	buf[0] = fs_buf_alloc(ffs->ffsbsize_b);
     
-    length = inode->size;
+	length = inode->size;
 	/* If we want the slack of the last fragment, then roundup */
-    if (flags & FS_FLAG_FILE_SLACK)
+	if (flags & FS_FLAG_FILE_SLACK)
 		length = roundup(length, fs->block_size);
 
-    /*
-     * Read the file blocks. First the direct blocks, then the indirect ones.
-     */     
-    for (n = 0; length > 0 && n < inode->direct_count; n++) {
-        retval = ffs_file_walk_direct(fs, buf[0], length, 
+	/*
+	 * Read the file blocks. First the direct blocks, then the indirect ones.
+	 */     
+	for (n = 0; length > 0 && n < inode->direct_count; n++) {
+		retval = ffs_file_walk_direct(fs, buf[0], length, 
 		  inode->direct_addr[n], flags, action, ptr);
 
 		if (!retval) {
@@ -829,9 +1034,9 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
 	}
 
 	fs_buf_free(buf[0]);
-    free((char *) buf);
+	free((char *) buf);
 
-    return;
+	return;
 }
 
 
@@ -846,22 +1051,26 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 {
 	int i;
 	time_t tmptime;
-	ffs_csum  *csum = NULL;
+	ffs_csum1  *csum1 = NULL;
 	ffs_cgd	 *cgd = NULL;
 
 	FFS_INFO *ffs = (FFS_INFO *) fs;
 	ffs_sb1 *sb1 = ffs->fs.sb1;
 	ffs_sb2 *sb2 = ffs->fs.sb2;
+	int flags;
 
 	fprintf(hFile, "FILE SYSTEM INFORMATION\n");
 	fprintf(hFile, "--------------------------------------------\n");   
 
-	if (ffs->ver == FFS_UFS1) {
+	if ((fs->ftype == FFS_1) ||
+	  (fs->ftype == FFS_1B)) {
 		fprintf(hFile, "File System Type: UFS 1\n");
 		tmptime = getu32(fs, sb1->wtime);
 		fprintf(hFile, "Last Written: %s", asctime(localtime(&tmptime)));
 		fprintf(hFile, "Last Mount Point: %s\n",
 		  sb1->last_mnt);
+
+		flags = sb1->fs_flags;
 	}
 	else {
 		fprintf(hFile, "File System Type: UFS 2\n");
@@ -873,6 +1082,43 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 		  sb2->volname);
 		fprintf(hFile, "System UID: %"PRIu64"\n",
 		  getu64(fs, sb2->swuid));
+		flags = getu32(fs, sb2->fs_flags);
+	}
+
+	if (flags) {
+		int cnt = 0;
+
+		fprintf(hFile, "Flags: ");
+
+		if (flags & FFS_SB_FLAG_UNCLEAN)	
+			fprintf(hFile, "%s Unclean",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_SOFTDEP)	
+			fprintf(hFile, "%s Soft Dependencies",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_NEEDFSCK)	
+			fprintf(hFile, "%s Needs fsck",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_INDEXDIR)	
+			fprintf(hFile, "%s Index directories",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_ACL)
+			fprintf(hFile, "%s ACLs",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_MULTILABEL)
+			fprintf(hFile, "%s TrustedBSD MAC Multi-label",
+			  (cnt++ == 0?"":","));
+
+		if (flags & FFS_SB_FLAG_UPDATED)
+			fprintf(hFile, "%s Updated Flag Location",
+			  (cnt++ == 0?"":","));
+
+		fprintf(hFile, "\n");
 	}
 
 
@@ -883,10 +1129,18 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "Inode Range: %lu - %lu\n", 
 	  (ULONG)fs->first_inum, (ULONG)fs->last_inum);
 	fprintf(hFile, "Root Directory: %lu\n", (ULONG)fs->root_inum);
-	fprintf(hFile, "Num of Avail Inodes: %"PRIu32"\n",
+	if ((fs->ftype == FFS_1) ||
+	  (fs->ftype == FFS_1B)) {
+		fprintf(hFile, "Num of Avail Inodes: %"PRIu32"\n",
 		       getu32(fs, sb1->cstotal.ino_free));	
-	fprintf(hFile, "Num of Directories: %"PRIu32"\n",
+		fprintf(hFile, "Num of Directories: %"PRIu32"\n",
 		       getu32(fs, sb1->cstotal.dir_num));	
+	} else {
+		fprintf(hFile, "Num of Avail Inodes: %"PRIu64"\n",
+		       getu64(fs, sb2->cstotal.ino_free));	
+		fprintf(hFile, "Num of Directories: %"PRIu64"\n",
+		       getu64(fs, sb2->cstotal.dir_num));	
+	}
 
 
 	fprintf(hFile, "\nCONTENT INFORMATION\n");
@@ -898,10 +1152,18 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "Block Size: %lu\n", (ULONG)ffs->ffsbsize_b);
 	fprintf(hFile, "Fragment Size: %lu\n", (ULONG)fs->block_size);
 
-	fprintf(hFile, "Num of Avail Full Blocks: %"PRIu32"\n",
+	if ((fs->ftype == FFS_1) ||
+	  (fs->ftype == FFS_1B)) {
+		fprintf(hFile, "Num of Avail Full Blocks: %"PRIu32"\n",
 		       getu32(fs, sb1->cstotal.blk_free));	
-	fprintf(hFile, "Num of Avail Fragments: %"PRIu32"\n",
+		fprintf(hFile, "Num of Avail Fragments: %"PRIu32"\n",
 		       getu32(fs, sb1->cstotal.frag_free));	
+	} else {
+		fprintf(hFile, "Num of Avail Full Blocks: %"PRIu64"\n",
+		       getu64(fs, sb2->cstotal.blk_free));	
+		fprintf(hFile, "Num of Avail Fragments: %"PRIu64"\n",
+		       getu64(fs, sb2->cstotal.frag_free));	
+	}
 
 	fprintf(hFile, "\nCYLINDER GROUP INFORMATION\n");
 	fprintf(hFile, "--------------------------------------------\n");
@@ -911,10 +1173,17 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "Fragments per group: %d\n", gets32(fs, sb1->cg_frag_num));
 
 
+	/* UFS 1 and 2 use the same ssize field  and use the same csum1 */
 	if (getu32(fs, sb1->cg_ssize_b)) {
-		csum = (ffs_csum *)mymalloc(getu32(fs, sb1->cg_ssize_b));
-		fs_read_random(fs, (char *) csum, getu32(fs, sb1->cg_ssize_b),
-		  getu32(fs, sb1->cg_saddr) * fs->block_size, "group descriptor");
+		csum1 = (ffs_csum1 *)mymalloc(getu32(fs, sb1->cg_ssize_b));
+		if ((fs->ftype == FFS_1) ||
+		  (fs->ftype == FFS_1B)) {
+			fs_read_random(fs, (char *) csum1, getu32(fs, sb1->cg_ssize_b),
+			  getu32(fs, sb1->cg_saddr) * fs->block_size, "group descriptor");
+		} else {
+			fs_read_random(fs, (char *) csum1, getu32(fs, sb2->cg_ssize_b),
+			  getu64(fs, sb2->cg_saddr) * fs->block_size, "group descriptor");
+		}
 	}
 
 	for (i = 0; i < gets32(fs, sb1->cg_num); i++) {
@@ -923,7 +1192,13 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 
 		fprintf(hFile, "\nGroup %d:\n", i);
 		if (cgd) {
-			tmptime = getu32(fs, cgd->wtime);
+			if ((fs->ftype == FFS_1) ||
+			  (fs->ftype == FFS_1B)) {
+				tmptime = getu32(fs, cgd->wtime);
+			} else {
+				ffs_cgd2 *cgd2 = (ffs_cgd2 *)cgd;
+				tmptime = (u_int32_t)getu64(fs, cgd2->wtime);
+			}
 			fprintf(hFile, "  Last Written: %s", asctime(localtime(&tmptime)));
 		}
 		
@@ -945,27 +1220,39 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 			fprintf(hFile, "    Boot Block: 0 - %lu\n",
 			  (ULONG)(15 * 512 / fs->block_size));
 
+
 			fprintf(hFile, "    Super Block: %lu - %lu\n",
 			  (ULONG)(16 * 512 / fs->block_size),
-			  (ULONG)(16 * 512 / fs->block_size) + 
-			  ((roundup(sizeof(ffs_sb1), fs->block_size) / fs->block_size) - 1));
+			  (ULONG)((16 * 512 / fs->block_size) + 
+			  ffs->ffsbsize_f - 1));
 		}
 
 		fprintf(hFile, "    Super Block: %lu - %lu\n",
 		  (ULONG)cgsblock_lcl(fs, sb1, i),
 		  (ULONG)(cgsblock_lcl(fs, sb1, i) + 
-		  ((roundup(sizeof(ffs_sb1), fs->block_size) / fs->block_size) - 1)));
+		  ffs->ffsbsize_f - 1));
+	
 
 		fprintf(hFile, "    Group Desc: %lu - %lu\n", 
 		  (ULONG)cgtod_lcl(fs, sb1, i),
 		  (ULONG)(cgtod_lcl(fs, sb1, i) + 
-		  ((roundup(sizeof(ffs_cgd), fs->block_size) / fs->block_size) - 1)));
+		  ffs->ffsbsize_f - 1));
 
-		fprintf(hFile, "    Inode Table: %lu - %lu\n", 
-		  (ULONG)cgimin_lcl(fs, sb1, i),
-		  (ULONG)(cgimin_lcl(fs, sb1, i) + 
-		  ((roundup(gets32(fs, sb1->cg_inode_num)*sizeof(ffs_inode1), fs->block_size) 
-		  / fs->block_size) - 1)));
+
+		if (fs->ftype == FFS_2) {
+			fprintf(hFile, "    Inode Table: %lu - %lu\n", 
+			  (ULONG)cgimin_lcl(fs, sb1, i),
+			  (ULONG)(cgimin_lcl(fs, sb1, i) + 
+			  ((roundup(gets32(fs, sb1->cg_inode_num)*sizeof(ffs_inode2), fs->block_size) 
+			  / fs->block_size) - 1)));
+		} 
+		else {
+			fprintf(hFile, "    Inode Table: %lu - %lu\n", 
+			  (ULONG)cgimin_lcl(fs, sb1, i),
+			  (ULONG)(cgimin_lcl(fs, sb1, i) + 
+			  ((roundup(gets32(fs, sb1->cg_inode_num)*sizeof(ffs_inode1), fs->block_size) 
+			  / fs->block_size) - 1)));
+		}
 
 		fprintf(hFile, "    Data Fragments: ");
 
@@ -983,19 +1270,18 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 		  ((ULONG)cgbase_lcl(fs, sb1, i + 1) - 1) : fs->last_block);
 
 
-		if ((csum) && ((i+1)*sizeof(ffs_csum) < getu32(fs, sb1->cg_ssize_b)) ) {
-			fprintf(hFile, "  Global Summary (from the super block):\n");
+		if ((csum1) && ((i+1)*sizeof(ffs_csum1) < getu32(fs, sb1->cg_ssize_b)) ) {
+			fprintf(hFile, "  Global Summary (from the superblock summary area):\n");
 			fprintf(hFile, "    Num of Dirs: %"PRIu32"\n", 
-				getu32(fs, &csum[i].dir_num));
+				getu32(fs, &csum1[i].dir_num));
 			fprintf(hFile, "    Num of Avail Blocks: %"PRIu32"\n", 
-				getu32(fs, &csum[i].blk_free));
+				getu32(fs, &csum1[i].blk_free));
 			fprintf(hFile, "    Num of Avail Inodes: %"PRIu32"\n", 
-				getu32(fs, &csum[i].ino_free));
+				getu32(fs, &csum1[i].ino_free));
 			fprintf(hFile, "    Num of Avail Frags: %"PRIu32"\n", 
-				getu32(fs, &csum[i].frag_free));
+				getu32(fs, &csum1[i].frag_free));
 		}
 
-
 		if (cgd) {
 			fprintf(hFile, "  Local Summary (from the group descriptor):\n");
 			fprintf(hFile, "    Num of Dirs: %"PRIu32"\n", 
@@ -1007,13 +1293,13 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 			fprintf(hFile, "    Num of Avail Frags: %"PRIu32"\n", 
 				getu32(fs, &cgd->cs.frag_free));
 
-			fprintf(hFile, "    Last Block Allocated: %"PRIu32"\n", 
+			fprintf(hFile, "    Last Block Allocated: %"PRIu64"\n", 
 				getu32(fs, &cgd->last_alloc_blk) +
-		  		cgbase_lcl(fs, sb1, i));
+		  		(uint64_t)cgbase_lcl(fs, sb1, i));
 
-			fprintf(hFile, "    Last Fragment Allocated: %"PRIu32"\n", 
+			fprintf(hFile, "    Last Fragment Allocated: %"PRIu64"\n", 
 				getu32(fs, &cgd->last_alloc_frag) +
-		  		cgbase_lcl(fs, sb1, i));
+		  		(uint64_t)cgbase_lcl(fs, sb1, i));
 
 			fprintf(hFile, "    Last Inode Allocated: %"PRIu32"\n", 
 				getu32(fs, &cgd->last_alloc_ino) + 
@@ -1090,15 +1376,15 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 
 	fprintf(hFile, "Group: %lu\n", (ULONG)ffs->cg_num);
 
-    if (fs_inode->link)
-        fprintf(hFile, "symbolic link to: %s\n", fs_inode->link);
+	if (fs_inode->link)
+		fprintf(hFile, "symbolic link to: %s\n", fs_inode->link);
 
-    fprintf(hFile, "uid / gid: %d / %d\n", 
+	fprintf(hFile, "uid / gid: %d / %d\n", 
 	  (int)fs_inode->uid, (int)fs_inode->gid);
 
 
-    make_ls(fs_inode->mode, ls, 12);
-    fprintf(hFile, "mode: %s\n", ls);
+	make_ls(fs_inode->mode, ls, 12);
+	fprintf(hFile, "mode: %s\n", ls);
 
 	fprintf(hFile, "size: %llu\n", (ULLONG) fs_inode->size);
 	fprintf(hFile, "num of links: %lu\n", (ULONG) fs_inode->nlink);
@@ -1106,27 +1392,99 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 
 	if (sec_skew != 0) {
 		fprintf(hFile, "\nAdjusted Inode Times:\n");
-        fs_inode->mtime -= sec_skew;
-        fs_inode->atime -= sec_skew; 
-        fs_inode->ctime -= sec_skew;
+		fs_inode->mtime -= sec_skew;
+		fs_inode->atime -= sec_skew; 
+		fs_inode->ctime -= sec_skew;
 
 		fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
 		fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
 		fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
 
-        fs_inode->mtime += sec_skew;
-        fs_inode->atime += sec_skew; 
-        fs_inode->ctime += sec_skew;
+		fs_inode->mtime += sec_skew;
+		fs_inode->atime += sec_skew; 
+		fs_inode->ctime += sec_skew;
 
 		fprintf(hFile, "\nOriginal Inode Times:\n");
 	}
-	else
+	else {
 		fprintf(hFile, "\nInode Times:\n");
+	}
 
 	fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
 	fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
 	fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
 
+	if (fs->ftype == FFS_2) {
+		ffs_inode2	*in = (ffs_inode2 *)ffs->dinode;
+		/* Are there extended attributes */
+		if (getu32(fs, in->di_extsize) > 0) {
+			ffs_extattr *ea;
+			u_int32_t size;
+			char name[257];
+			FS_BUF *fs_buf = fs_buf_alloc(ffs->ffsbsize_b);
+
+			size = getu32(fs, in->di_extsize);
+			fprintf(hFile, "\nExtended Attributes:\n");
+			fprintf(hFile, "Size: %"PRIu32" (%"PRIu64", %"PRIu64")\n", 
+			  size, getu64(fs, in->di_extb[0]), 
+			  getu64(fs, in->di_extb[1]));
+
+
+			/* Process first block */
+			// @@@ Incorporate values into this as well
+			if ((getu64(fs, in->di_extb[0]) >= fs->first_block) && 
+			  (getu64(fs, in->di_extb[0]) <= fs->last_block)) {
+				int end;				
+
+				fs_read_block(fs, fs_buf, ffs->ffsbsize_b, 
+				  getu64(fs, in->di_extb[0]),
+			      	  "extended attribute");
+
+				ea = (ffs_extattr *)fs_buf->data;
+
+				if (size > ffs->ffsbsize_b) {
+					end = (uintptr_t)ea + ffs->ffsbsize_b;
+					size -= ffs->ffsbsize_b;
+				}
+				else {
+					end = (uintptr_t)ea + size;
+					size = 0;
+				}
+
+				for ( ;(uintptr_t)ea < end; 
+				  ea = (ffs_extattr *)((uintptr_t)ea + getu32(fs, ea->reclen))) {
+					memcpy (name, ea->name, ea->nlen);
+					name[ea->nlen] = '\0';
+					fprintf(hFile, "%s\n", name);
+				}
+			}
+			if ((getu64(fs, in->di_extb[1]) >= fs->first_block) && 
+			  (getu64(fs, in->di_extb[1]) <= fs->last_block)) {
+				int end;				
+
+				fs_read_block(fs, fs_buf, ffs->ffsbsize_b, 
+				  getu64(fs, in->di_extb[1]),
+			      	  "extended attribute");
+
+				ea = (ffs_extattr *)fs_buf->data;
+	
+				if (size > ffs->ffsbsize_b)
+					end = (uintptr_t)ea + ffs->ffsbsize_b;
+				else 
+					end = (uintptr_t)ea + size;
+
+				for ( ;(uintptr_t)ea < end; 
+				  ea = (ffs_extattr *)((uintptr_t)ea + getu32(fs, ea->reclen))) {
+					memcpy (name, ea->name, ea->nlen);
+					name[ea->nlen] = '\0';
+					fprintf(hFile, "%s\n", name);
+				}
+			}
+		}
+	}
+
+
+
 	/* A bad hack to force a specified number of blocks */
 	if (numblock > 0)
 		fs_inode->size = numblock * ffs->ffsbsize_b;
@@ -1149,11 +1507,15 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
         printidx = 0;
 
         for (i = 0; i < indir_idx; i++) {
-            fprintf(hFile, "%lu ", (unsigned long) indirl[i]);
-            if (++printidx == WIDTH) {
-                fprintf(hFile, "\n");
-                printidx = 0;
-            }
+		int a;
+		/* Cycle through the fragments in the block */
+		for (a = 0; a < ffs->ffsbsize_f; a++) {
+			fprintf(hFile, "%lu ", (unsigned long) indirl[i] + a);
+			if (++printidx == WIDTH) {
+				fprintf(hFile, "\n");
+				printidx = 0;
+			}
+		}
         }
         if (printidx != 0)
             fprintf(hFile, "\n");
@@ -1162,6 +1524,28 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	return;
 }
 
+void
+ffs_jopen (FS_INFO *fs, u_int32_t inum)
+{
+        fprintf(stderr, "Error: UFS does not have a journal\n");
+        exit(1);
+}
+
+void
+ffs_jentry_walk (FS_INFO *fs, int flags, FS_JENTRY_WALK_FN action, char *ptr)
+{
+        fprintf(stderr, "Error: UFS does not have a journal\n");
+        exit(1);
+}
+
+
+void
+ffs_jblk_walk (FS_INFO *fs, u_int32_t start, u_int32_t end, int flags, FS_JBLK_WALK_FN action, char *ptr)
+{
+        fprintf(stderr, "Error: UFS does not have a journal\n");
+        exit(1);
+}
+
 
 
 /* ffs_close - close a fast file system */
@@ -1202,12 +1586,13 @@ ffs_open(const char *name, unsigned char ftype)
 	/*
 	 * Try UFS 1 first
 	 */
-	/* ffs_sb2 is bigger, so allocate space for it */
-	len = roundup(sizeof(ffs_sb2), FFS_DEV_BSIZE);
+	/* Both sbs are the same size */
+	len = roundup(sizeof(ffs_sb1), FFS_DEV_BSIZE);
 	ffs->fs.sb1 = (ffs_sb1 *) mymalloc(len);
 
 	if (LSEEK(fs->fd, UFS1_SBOFF, SEEK_SET) != UFS1_SBOFF)
 		error("%s: lseek: %m", myname);
+
 	if (read(fs->fd, (char *) ffs->fs.sb1, sizeof(ffs_sb1)) != sizeof(ffs_sb1))
 		error("%s: read superblock: %m", name);
 
@@ -1231,41 +1616,40 @@ ffs_open(const char *name, unsigned char ftype)
 				error("Error: %s is not a FFS file system", name);
 			}
 		}
-		ffs->ver = FFS_UFS2;
-	}
-	else {
-		ffs->ver = FFS_UFS1;
+		if (ftype == FFS_1B)
+			error("UFS2 is not supported for this OS");
+
+		fs->ftype = FFS_2;
 	}
 
 
 	/*
 	 * Translate some filesystem-specific information to generic form.
 	 */
-	if (ffs->ver == FFS_UFS1) {
+	if (fs->ftype == FFS_2) {
+		fs->inum_count = gets32(fs, ffs->fs.sb2->cg_num) * gets32(fs, ffs->fs.sb2->cg_inode_num);
+	} 
+	else {	
 		fs->inum_count = gets32(fs, ffs->fs.sb1->cg_num) * gets32(fs, ffs->fs.sb1->cg_inode_num);
 	} 
-	else {
-		fs->inum_count = gets32(fs, ffs->fs.sb2->cg_num) * gets32(fs, ffs->fs.sb2->cg_inode_num);
-
-	}
 
 	fs->root_inum = FFS_ROOTINO;
 	fs->first_inum = FFS_FIRSTINO;
 	fs->last_inum = fs->inum_count - 1;
 
 
-	if (ffs->ver == FFS_UFS1) {
-		fs->block_count = gets32(fs, ffs->fs.sb1->frag_num);
-		fs->block_size = gets32(fs, ffs->fs.sb1->fsize_b);
-		ffs->ffsbsize_b = gets32(fs, ffs->fs.sb1->bsize_b);
-		ffs->ffsbsize_f = gets32(fs, ffs->fs.sb1->bsize_frag);
-	}
-	else {
+	if (fs->ftype == FFS_2) {
 		fs->block_count = gets64(fs, ffs->fs.sb2->frag_num);
 		fs->block_size = gets32(fs, ffs->fs.sb2->fsize_b);
 		ffs->ffsbsize_b = gets32(fs, ffs->fs.sb2->bsize_b);
 		ffs->ffsbsize_f = gets32(fs, ffs->fs.sb2->bsize_frag);
 	}
+	else {
+		fs->block_count = gets32(fs, ffs->fs.sb1->frag_num);
+		fs->block_size = gets32(fs, ffs->fs.sb1->fsize_b);
+		ffs->ffsbsize_b = gets32(fs, ffs->fs.sb1->bsize_b);
+		ffs->ffsbsize_f = gets32(fs, ffs->fs.sb1->bsize_frag);
+	}
 
 	fs->first_block = 0;
 	fs->last_block = fs->block_count - 1;
@@ -1275,9 +1659,9 @@ ffs_open(const char *name, unsigned char ftype)
 	/*
 	 * Other initialization: caches, callbacks.
 	 */
-	ffs->cg_buf = 0;
+	ffs->cg_buf = NULL;
 	ffs->cg_num = -1;
-	ffs->dino_buf = 0;
+	ffs->dino_buf = NULL;
 	fs->seek_pos = -1;
 
 	fs->inode_walk = ffs_inode_walk;
@@ -1289,13 +1673,20 @@ ffs_open(const char *name, unsigned char ftype)
 	fs->fscheck = ffs_fscheck;
 	fs->istat = ffs_istat;
 	fs->close = ffs_close;
+	fs->jblk_walk = ffs_jblk_walk;
+	fs->jentry_walk = ffs_jentry_walk;
+	fs->jopen = ffs_jopen;
+	fs->journ_inum = 0;
 
 	/* allocate the dinode buffer */
-	if (ffs->ver == FFS_UFS1) 
-		ffs->dinode = (char *) mymalloc(sizeof(ffs_inode1));
+	if (fs->ftype == FFS_2) 
+		ffs->dinode = (char *) mymalloc(sizeof(ffs_inode2));
 	else
 		ffs->dinode = (char *) mymalloc(sizeof(ffs_inode1));
 
+
+
+
 	/*
 	 * Print some stats.
 	 */
