@@ -73,7 +73,8 @@ ffs_dent_copy(FFS_INFO *ffs, char *ffs_dent, FS_DENT *fs_dent)
 	FS_INFO *fs = &(ffs->fs_info);
 
 	/* this one has the type field */
-	if (fs->ftype == FFS_1) {
+	if ((fs->ftype == FFS_1) ||
+	  (fs->ftype == FFS_2)) {
 		ffs_dentry1  *dir = (ffs_dentry1 *)ffs_dent;
 
 		fs_dent->inode = getu32(fs, dir->d_ino);
@@ -93,8 +94,7 @@ ffs_dent_copy(FFS_INFO *ffs, char *ffs_dent, FS_DENT *fs_dent)
 		fs_dent->ent_type = dir->d_type;
 
 	}
-	/* FFS_2 */
-	else {
+	else if (fs->ftype == FFS_1B) {
 		ffs_dentry2  *dir = (ffs_dentry2 *)ffs_dent;
 
 		fs_dent->inode = getu32(fs, dir->d_ino);
@@ -110,6 +110,9 @@ ffs_dent_copy(FFS_INFO *ffs, char *ffs_dent, FS_DENT *fs_dent)
 
 		fs_dent->ent_type = FS_DENT_UNDEF;
 	}
+	else {
+		error("dent_copy: Unknown FFS Type");
+	}
 
 	/* copy the path data */
 	fs_dent->path = dirs;
@@ -141,7 +144,7 @@ ffs_dent_parse_block (FFS_INFO *ffs, char *buf, int len,
   int flags, FS_DENT_WALK_FN action, char *ptr) 
 {
 	unsigned int idx;
-	int inode, dellen = 0, reclen = 0;
+	int inode = 0, dellen = 0, reclen = 0;
 	int minreclen = 4;
 	FS_INFO *fs = &(ffs->fs_info);
 
@@ -154,19 +157,20 @@ ffs_dent_parse_block (FFS_INFO *ffs, char *buf, int len,
 	** recorded length so we can view the deleted entries 
 	*/
 	for (idx = 0; idx <= len - FFS_DIRSIZ_lcl(1); idx += minreclen) {
-		int namelen, myflags;
+		int namelen = 0, myflags = 0;
 
 		dirPtr = (char *)&buf[idx];
 
 		/* copy to local variables */
-		if (fs->ftype == FFS_1) {
+		if ((fs->ftype == FFS_1) ||
+		  (fs->ftype == FFS_2)) {
 			ffs_dentry1 *dir = (ffs_dentry1 *)dirPtr;
 			inode = getu32(fs, dir->d_ino);
 			namelen = dir->d_namlen;
 			reclen = getu16(fs, dir->d_reclen);
 		}
-		/* FFS_2 */
-		else {
+		/* FFS_1B */
+		else if (fs->ftype == FFS_1B) {
 			ffs_dentry2 *dir = (ffs_dentry2 *)dirPtr;
 			inode = getu32(fs, dir->d_ino);
 			namelen = getu16(fs, dir->d_namlen);
@@ -273,7 +277,7 @@ ffs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 { 
 	int len = (dirleft < size) ? dirleft : size;
 	memcpy (curdirptr, buf, len);
-	curdirptr = (char *) ((int)curdirptr + len);
+	curdirptr = (char *) ((uintptr_t)curdirptr + len);
 	dirleft -= len;
 	return (dirleft) ? WALK_CONT : WALK_STOP;
 }
