@@ -281,7 +281,7 @@ fs_data_put_name(FS_DATA *data, char *name)
  */
 FS_DATA *
 fs_data_put_str(FS_DATA *data_head, char *name, u_int32_t type, u_int16_t id,
-  DADDR_T *addr, int len)
+  void *addr, int len)
 {
 
 	FS_DATA *data;
@@ -300,12 +300,13 @@ fs_data_put_str(FS_DATA *data_head, char *name, u_int32_t type, u_int16_t id,
 
 	fs_data_put_name(data, name);
     
-    if (data->buflen < len) {
-        data->buf = (u_int8_t *)myrealloc((char *)data->buf, len);
-        data->buflen = len;
-    }
+	if (data->buflen < len) {
+ 		data->buf = (u_int8_t *)myrealloc((char *)data->buf, len);
+		data->buflen = len;
+	}
+
 	memset(data->buf, 0, data->buflen);
-    memcpy(data->buf, addr, len);
+	memcpy(data->buf, addr, len);
 	data->size = len;
 
 	return data_head;
