@@ -7,7 +7,7 @@ extern FS_DATA *fs_data_getnew_attr(FS_DATA *, u_int8_t);
 extern void fs_data_clear_list(FS_DATA *);
 
 extern FS_DATA *fs_data_put_str(FS_DATA *, char *, u_int32_t, u_int16_t, 
-  DADDR_T *, int);
+  void *, int);
 
 extern FS_DATA *fs_data_put_run(FS_DATA *, u_int64_t, u_int64_t,
   FS_DATA_RUN *, char *, u_int32_t, u_int16_t, u_int64_t, u_int8_t);
