@@ -138,7 +138,7 @@ FS_INFO *fs_open(const char *path, const char *type)
 	switch (ftype & FSMASK) {
 		case FFS_TYPE:
 			return ffs_open(path, ftype);
-		case EXT2FS_TYPE:
+		case EXTxFS_TYPE:
 			return ext2fs_open(path, ftype);
 		case FATFS_TYPE:
 			return fatfs_open(path, ftype);
