@@ -59,6 +59,7 @@ typedef struct FS_DENT FS_DENT;
 typedef struct FS_DATA FS_DATA;
 typedef struct FS_DATA_RUN FS_DATA_RUN;
 typedef struct FS_NAME FS_NAME;
+typedef struct FS_JENTRY FS_JENTRY;
 
 /* Flags for the return value of inode_walk, block_walk, and dent_walk
  * actions
@@ -72,49 +73,62 @@ typedef u_int8_t (*FS_BLOCK_WALK_FN) (FS_INFO *, DADDR_T, char *, int, char *);
 typedef u_int8_t (*FS_DENT_WALK_FN) (FS_INFO *, FS_DENT *, int, char *);
 typedef u_int8_t (*FS_FILE_WALK_FN) (FS_INFO *, DADDR_T, char *, int, int, char *);
 
+typedef u_int8_t (*FS_JBLK_WALK_FN) (FS_INFO *, char *, int, char *);
+typedef u_int8_t (*FS_JENTRY_WALK_FN) (FS_INFO *, FS_JENTRY *, int, char *);
+
 
 /***************************************************************
  * FS_INFO: Allocated when an image is opened
  */
 struct FS_INFO {
-    int     fd;				/* open raw device */
-    OFF_T   seek_pos;			/* current seek position */
+	int     fd;				/* open raw device */
+	OFF_T   seek_pos;			/* current seek position */
 
 	/* meta data */
-    INUM_T  inum_count;			/* number of inodes */
-    INUM_T  root_inum;			/* root inode */
+	INUM_T  inum_count;			/* number of inodes */
+	INUM_T  root_inum;			/* root inode */
 	INUM_T	first_inum;			/* first valid inode */
-    INUM_T  last_inum;			/* LINUX starts at 1 */
+	INUM_T  last_inum;			/* LINUX starts at 1 */
 
 	/* content */
 	DADDR_T block_count;		/* number of blocks */
 	DADDR_T first_block;		/* in case start at 1 */
 	DADDR_T last_block;			/* in case start at 1 */
 	int     block_size;			/* block size in bytes */
-	int		dev_bsize;			/* value of DEV_BSIZE constant */
+	int	dev_bsize;			/* value of DEV_BSIZE constant */
 
+	/* Journal */
+	INUM_T	journ_inum;		/* Inode of journal */
 
 	unsigned char ftype;		/* type of file system */
 	unsigned char	flags;		/* flags for image, see below */
 
 
 	/* file system specific function pointers */
-    void    (*block_walk) (FS_INFO *, DADDR_T, DADDR_T, int, 
+	void    (*block_walk) (FS_INFO *, DADDR_T, DADDR_T, int, 
 	  FS_BLOCK_WALK_FN, char *);
 
-    void    (*inode_walk) (FS_INFO *, INUM_T, INUM_T, int, 
+	void    (*inode_walk) (FS_INFO *, INUM_T, INUM_T, int, 
 	  FS_INODE_WALK_FN, char *);
-    FS_INODE *(*inode_lookup) (FS_INFO *, INUM_T);
+	FS_INODE *(*inode_lookup) (FS_INFO *, INUM_T);
 	void	(*istat) (FS_INFO *, FILE *, INUM_T, int, int32_t);
-    void    (*file_walk) (FS_INFO *, FS_INODE *, u_int32_t, u_int16_t, 
+
+	void    (*file_walk) (FS_INFO *, FS_INODE *, u_int32_t, u_int16_t, 
 	  int, FS_FILE_WALK_FN, char *);
 
-    void    (*dent_walk) (FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
+	void    (*dent_walk) (FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
+
+	void	(*jopen) (FS_INFO *, u_int32_t);
+	void    (*jblk_walk) (FS_INFO *, u_int32_t, u_int32_t, int, 
+	  FS_JBLK_WALK_FN, char *);
+	void    (*jentry_walk) (FS_INFO *, int, 
+	  FS_JENTRY_WALK_FN, char *);
+
 
 	void	(*fsstat) (FS_INFO *, FILE *);
 	void	(*fscheck) (FS_INFO *, FILE *);
 
-    void    (*close) (FS_INFO *);
+	void    (*close) (FS_INFO *);
 };
 
 /* flag for FS_INFO flags */
@@ -345,6 +359,18 @@ extern char fs_inode_str[FS_INODE_MAX_STR][2];
 
 
 
+/**************** Journal Stuff **********************/
+struct FS_JENTRY {
+	u_int32_t	jblk;	/* journal block address */
+	u_int32_t	fsblk;	/* fs block that journal entry is about */
+};
+
+
+
+
+
+
+
 /* function decs */
 extern FS_DENT *fs_dent_alloc(ULONG);
 extern FS_DENT *fs_dent_realloc(FS_DENT *, ULONG);
