@@ -36,7 +36,7 @@ FS_TYPES fs_open_table[] = {
 	{"ntfs", NTFS, "NTFS"},
 	{"openbsd", FFS_1, "OpenBSD FFS"},
 	{"raw", RAW, "Raw Data"},
-	{"solaris", FFS_2, "Solaris FFS"},
+	{"solaris", FFS_1B, "Solaris FFS"},
 	{"swap", SWAP, "Swap Space"},
 	{0},
 };
