@@ -27,8 +27,9 @@ extern void fs_print_types();
 /* FFS */
 #define FFS_TYPE		0x10
 
-#define FFS_1			0x11	/* FreeBSD, OpenBSD, BSDI ... */
-#define FFS_2			0x12	/* Solaris (no type) */
+#define FFS_1			0x11	/* UFS1 - FreeBSD, OpenBSD, BSDI ... */
+#define FFS_1B			0x12	/* Solaris (no type) */
+#define FFS_2			0x13	/* UFS2 - FreeBSD, NetBSD */
 
 
 #define	EXTxFS_TYPE		0x20
