@@ -30,13 +30,13 @@ extern char *progname;
 static void 
 usage()
 {
-    printf("usage: %s [-hHsvV] [-f fstype] device [inum[-typ[-id]] ...]\n", progname);
+	printf("usage: %s [-hHsvV] [-f fstype] device [inum[-typ[-id]] ...]\n", progname);
 	printf("\t-h: Do not display holes in sparse files\n");
 	printf("\t-r: Recover deleted file\n");
 	printf("\t-s: Display slack space at end of file\n");
 	printf("\t-v: verbose to stderr\n");
 	printf("\t-V: Print version\n");
-    printf("\t-f fstype: Image file system type\n");
+	printf("\t-f fstype: Image file system type\n");
 	printf("Supported file system types:\n");
 	fs_print_types();
 
@@ -61,15 +61,15 @@ icat_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 int
 main(int argc, char **argv)
 {
-    FS_INFO *fs;
-    char   	*cp;
-    INUM_T  inum;
-	int		flags = 0;
-    int     ch;
-    char   	*fstype = DEF_FSTYPE;
+	FS_INFO *fs;
+	char   	*cp;
+	INUM_T  inum;
+	int	flags = 0;
+	int     ch;
+	char   	*fstype = DEF_FSTYPE;
 	FS_INODE *inode;
 
-    progname = argv[0];
+	progname = argv[0];
 
     while ((ch = getopt(argc, argv, "f:hrsvV")) > 0) {
 	switch (ch) {
@@ -102,9 +102,9 @@ main(int argc, char **argv)
 
     fs = fs_open(argv[optind], fstype);
 
-    while (argv[++optind]) {
+	while (argv[++optind]) {
 		int type = 0;
-		int id = 0;
+		int id = 0, id_used = 0;
 		char 	*dash;
 
 		/* simple inode usage */
@@ -128,6 +128,7 @@ main(int argc, char **argv)
 				dash2++;
 
 				id = STRTOUL(dash2, &cp, 0);
+				id_used = 1;
 				if (*cp || cp == dash2)
 					usage();
 			}
@@ -145,10 +146,14 @@ main(int argc, char **argv)
 		if (!inode)
 			error ("error getting inode");
 
-		fs->file_walk(fs, inode, type, id, flags, icat_action, "");
+		if (id_used)
+			fs->file_walk(fs, inode, type, id, flags, icat_action, "");
+		/* If the id value was not used, then set the flag accordingly so the '0' value is ignored */
+		else 
+			fs->file_walk(fs, inode, type, id, flags | FS_FLAG_FILE_NOID, icat_action, "");
 
 		fs_inode_free(inode);
-    }
-    fs->close(fs);
-    exit(0);
+	}
+	fs->close(fs);
+	exit(0);
 }
