@@ -222,7 +222,7 @@ find_inode_path (FS_INFO *fs, FS_DENT *fs_dent, int flags, char *ptr)
 			sh_ptr++;
 
 			/* Length of long name - 2 for ' (' */
-			long_len = (int)sh_ptr - (int)fs_dent->name - 2;
+			long_len = (uintptr_t)sh_ptr - (uintptr_t)fs_dent->name - 2;
 
 			/* Length of Short name - 3 for ' (' and ')' */
 			sh_len = strlen (fs_dent->name) - long_len - 3;
