@@ -2,7 +2,7 @@
 ** The Sleuth Kit 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -250,7 +250,7 @@ print_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 		fs_inode->ctime -= sec_skew;
 	}
     printf("%lu|%c|%d|%d|%lu|%lu|%lu",
-	   (ULONG) inum, (flags & FS_FLAG_ALLOC) ? 'a' : 'f',
+	   (ULONG) inum, (flags & FS_FLAG_META_ALLOC) ? 'a' : 'f',
 	   (int) fs_inode->uid, (int) fs_inode->gid,
 	   (ULONG) fs_inode->mtime, (ULONG) fs_inode->atime,
 	   (ULONG) fs_inode->ctime);
@@ -295,7 +295,7 @@ print_inode_mac(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 	printf("0|<%s-%s%s%s-%lu>|0|%lu|%d|", image, 
 	  (fs_inode->name)?fs_inode->name->name:"",
 	  (fs_inode->name)?"-":"",
-	  (flags & FS_FLAG_ALLOC)?"alive":"dead", (ULONG)inum, 
+	  (flags & FS_FLAG_META_ALLOC)?"alive":"dead", (ULONG)inum, 
 	  (ULONG)inum, (int)fs_inode->mode);
 
 	/* Print the "ls" mode in ascii format */
@@ -359,7 +359,7 @@ int     main(int argc, char **argv)
 		argflags |= ARG_M;
 		break;
 	case 'o':
-		flags |= (FS_FLAG_ALLOC | FS_FLAG_UNLINK);
+		flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNLINK);
 		argflags |= ARG_O;
 	    break;
 	case 'r':
@@ -380,28 +380,28 @@ int     main(int argc, char **argv)
 	     * Provide fine controls to tweak one feature at a time.
 	     */
 	case 'a':
-	    flags |= FS_FLAG_ALLOC;
-	    flags &= ~FS_FLAG_UNALLOC;
+	    flags |= FS_FLAG_META_ALLOC;
+	    flags &= ~FS_FLAG_META_UNALLOC;
 	    break;
 	case 'A':
-	    flags |= FS_FLAG_UNALLOC;
-	    flags &= ~FS_FLAG_ALLOC;
+	    flags |= FS_FLAG_META_UNALLOC;
+	    flags &= ~FS_FLAG_META_ALLOC;
 	    break;
 	case 'l':
-	    flags |= FS_FLAG_LINK;
-	    flags &= ~FS_FLAG_UNLINK;
+	    flags |= FS_FLAG_META_LINK;
+	    flags &= ~FS_FLAG_META_UNLINK;
 	    break;
 	case 'L':
-	    flags |= FS_FLAG_UNLINK;
-	    flags &= ~FS_FLAG_LINK;
+	    flags |= FS_FLAG_META_UNLINK;
+	    flags &= ~FS_FLAG_META_LINK;
 	    break;
 	case 'z':
-	    flags |= FS_FLAG_UNUSED;
-	    flags &= ~FS_FLAG_USED;
+	    flags |= FS_FLAG_META_UNUSED;
+	    flags &= ~FS_FLAG_META_USED;
 	    break;
 	case 'Z':
-	    flags |= FS_FLAG_USED;
-	    flags &= ~FS_FLAG_UNUSED;
+	    flags |= FS_FLAG_META_USED;
+	    flags &= ~FS_FLAG_META_UNUSED;
 	    break;
 	}
     }
@@ -438,9 +438,9 @@ int     main(int argc, char **argv)
 	if ((argflags & ARG_R) || (flags == 0)) {
 		if (((fs->ftype & FSMASK) == NTFS_TYPE) ||
 		  ((fs->ftype & FSMASK) == FATFS_TYPE)) 
-			flags |= (FS_FLAG_USED | FS_FLAG_UNALLOC);
+			flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNALLOC);
 		else 
-			flags |= (FS_FLAG_USED | FS_FLAG_UNLINK);
+			flags |= (FS_FLAG_META_USED | FS_FLAG_META_UNLINK);
 	}
 
 	/* If neither of the flags in a family are set, then set both 
@@ -448,12 +448,14 @@ int     main(int argc, char **argv)
      * Apply rules for default settings. Assume a "don't care" condition when
      * nothing is explicitly selected from a specific feature category.
 	 */
-    if ((flags & (FS_FLAG_ALLOC | FS_FLAG_UNALLOC)) == 0)
-		flags |= FS_FLAG_ALLOC | FS_FLAG_UNALLOC;
-    if ((flags & (FS_FLAG_LINK | FS_FLAG_UNLINK)) == 0) 
-		flags |= FS_FLAG_LINK | FS_FLAG_UNLINK;
-    if ((flags & (FS_FLAG_USED | FS_FLAG_UNUSED)) == 0)
-		flags |= FS_FLAG_USED | FS_FLAG_UNUSED;
+    if ((flags & (FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC)) == 0)
+		flags |= FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC;
+
+    if ((flags & (FS_FLAG_META_LINK | FS_FLAG_META_UNLINK)) == 0) 
+		flags |= FS_FLAG_META_LINK | FS_FLAG_META_UNLINK;
+
+    if ((flags & (FS_FLAG_META_USED | FS_FLAG_META_UNUSED)) == 0)
+		flags |= FS_FLAG_META_USED | FS_FLAG_META_UNUSED;
 
 
     /*
