@@ -270,7 +270,7 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 		  mftaddr_len, mftaddr_b, "mft read");
 
 		/* read the second part into mft */
-		fs_read_random(&ntfs->fs_info, (char *)((int)mft + mftaddr_len), 
+		fs_read_random(&ntfs->fs_info, (char *)((uintptr_t)mft + mftaddr_len), 
 		  ntfs->mft_rsize_b - mftaddr_len, mftaddr2_b, "mft read");
 	}
 	else {	
@@ -280,7 +280,7 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 	}
 
 	/* if we are saving into the NTFS_INFO structure, assign mnum too */
-	if ((int)mft == (int)ntfs->mft)
+	if ((uintptr_t)mft == (uintptr_t)ntfs->mft)
 		ntfs->mnum = mftnum;
 
 	/* Sanity Check */
@@ -311,10 +311,9 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 	if ((getu16(fs, mft->upd_cnt) - 1) * ntfs->ssize_b > ntfs->mft_rsize_b)
 		error ("More Update Sequence Entries than MFT size");
 
-
 	/* Apply the update sequence structure template */
-	upd = (ntfs_upd *)((int)mft + getu16(fs, mft->upd_off));
-	
+	upd = (ntfs_upd *)((uintptr_t)mft + getu16(fs, mft->upd_off));
+
 	/* Get the sequence value that each 16-bit value should be */
 	sig_seq = getu16(fs, upd->upd_val);
 
@@ -326,18 +325,18 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 		u_int8_t *new, *old;
 
 		/* get the current sequence value */
-		u_int16_t cur_seq = getu16(fs, (int)mft + offset);
+		u_int16_t cur_seq = getu16(fs, (uintptr_t)mft + offset);
 
 		if (cur_seq != sig_seq) {
 			/* get the replacement value */
 			u_int16_t cur_repl =
 			  getu16(fs, &upd->upd_seq + (i-1) * 2);
-			error ("Incorrect update sequence value in MFT entry\nUpdate Value: 0x%x Actual Value: 0x%x Replacement Value: 0x%x\nThis is typically because of a corrupted entry",
+			error ("Incorrect update sequence value in MFT entry\nSignature Value: 0x%x Actual Value: 0x%x Replacement Value: 0x%x\nThis is typically because of a corrupted entry",
 			  sig_seq, cur_seq, cur_repl);
 		}
 
 		new = &upd->upd_seq + (i-1) * 2;
-		old = (u_int8_t *)(int)mft + offset;
+		old = (u_int8_t *)((uintptr_t)mft + offset);
 
 		if (verbose)
 			fprintf(logfp, 
@@ -362,6 +361,15 @@ is_clust_alloc (NTFS_INFO *ntfs, u_int32_t addr)
 	int	bits_p_clust, base, b;
 	bits_p_clust = 8 * ntfs->fs_info.block_size;
 
+	/* While we are loading the MFT, assume that everything
+	 * is allocated.  This should only be needed when we are
+	 * dealing with an attribute list ... 
+	 */
+	if (loading_the_MFT == 1)
+		return 1;
+	else if (ntfs->bmap == NULL)
+		error ("Bitmap pointer is null: %"PRIu32"\n", addr);
+
 	/* Is the cluster too big? */
 	if (addr > ntfs->fs_info.last_block)
 		error ("is_clust_alloc: cluster too large");
@@ -529,7 +537,7 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 
 
 		/* Advance run */
-		run = (ntfs_runlist *) ((int)run + 
+		run = (ntfs_runlist *) ((uintptr_t)run + 
 		   (1 + NTFS_RUNL_LENSZ(run) + NTFS_RUNL_OFFSZ(run) ) );
 
 	}
@@ -600,7 +608,7 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 
 		if ( ((flags & FS_FLAG_FILE_NOABORT) == 0) && 
 		  (fs_data->flags & FS_DATA_COMP) ) {
-			error ("ERROR: TSK Cannot process compressed files");
+			error ("ERROR: TSK Cannot uncompress NTFS compressed files - aborting");
 		}
 
 
@@ -738,7 +746,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
   int len)
 {
 	ntfs_attr 	*attr = attrseq;
-	FS_DATA		*fs_data_attrl = NULL, *fs_data;
+	FS_DATA		*fs_data_attrl = NULL, *fs_data = NULL;
 	char		name[NTFS_MAXNAMLEN+1];
 	u_int64_t	runlen;
 	FS_INFO 	*fs = (FS_INFO *)&ntfs->fs_info;
@@ -748,11 +756,11 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		  (ULONG)ntfs->mnum);
 
 	/* Cycle through the list of attributes */
-	for ( ; ((int)attr >= (int)attrseq) && 
-			((int)attr <= ((int)attrseq + len)) && 
-			(getu32(fs, attr->len) > 0 &&
-			(getu32(fs, attr->type) != 0xffffffff));
-	        (int)attr += getu32(fs, attr->len)) 
+	for ( ; ((uintptr_t)attr >= (uintptr_t)attrseq) && 
+		((uintptr_t)attr <= ((uintptr_t)attrseq + len)) && 
+		(getu32(fs, attr->len) > 0 &&
+		(getu32(fs, attr->type) != 0xffffffff));
+	        (uintptr_t)attr += getu32(fs, attr->len)) 
 	{
 		/* Get the type of this attribute */
 		u_int32_t	type = getu32(fs, attr->type);
@@ -768,7 +776,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			/* Copy the name into ASCII */
 			if (attr->nlen)
 				uni2ascii(
-				  (char *)(int)attr + getu16(fs, attr->name_off), 
+				  (char *)((uintptr_t)attr + getu16(fs, attr->name_off)), 
 				  attr->nlen, name, sizeof(name));
 
 			/* Call the unnamed $Data attribute, $Data */
@@ -786,7 +794,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			/* Add this resident stream to the fs_inode->attr list */
 			fs_inode->attr = fs_data_put_str(fs_inode->attr, name, 
 			  type, getu16(fs, attr->id),
-			  (DADDR_T *)((int)attr + getu16(fs, attr->c.r.soff)), 
+			  (void *)((uintptr_t)attr + getu16(fs, attr->c.r.soff)), 
 			  getu32(fs, attr->c.r.ssize));
 
 		}
@@ -797,11 +805,12 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		else {
 			FS_DATA_RUN *fs_data_run;
 			u_int8_t	data_flag = 0;
+			int	id;
 
 			/* Copy the name */
 			if (attr->nlen)
 				uni2ascii(
-				  (char *)(int)attr + getu16(fs, attr->name_off), 
+				  (char *)((uintptr_t)attr + getu16(fs, attr->name_off)), 
 				  attr->nlen, name, sizeof(name));
 
 			else if (type == NTFS_ATYPE_DATA)
@@ -809,14 +818,17 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			else
 				strncpy(name, "N/A", NTFS_MAXNAMLEN+1);
 
+
+			id = getu16(fs, attr->id);
+
 			if (verbose)
 				fprintf(logfp, 
-				  "ntfs_proc_attrseq: Non-Resident Attribute in %lu Type: %lu Id: %lu Name: %s\n", 
-				  (ULONG)ntfs->mnum, (ULONG)type, (ULONG)getu16(fs, attr->id), name);
+				  "ntfs_proc_attrseq: Non-Resident Attribute in %lu Type: %lu Id: %lu Name: %s  Start VCN: %"PRIu64"\n", 
+				  (ULONG)ntfs->mnum, (ULONG)type, (ULONG)id, name, getu64(fs, attr->c.nr.start_vcn));
 
 			/* convert the run to generic form */
 			fs_data_run = ntfs_make_data_run(ntfs, 
-			  (ntfs_runlist *) ((int)attr + getu16(fs, attr->c.nr.run_off)),
+			  (ntfs_runlist *) ((uintptr_t)attr + getu16(fs, attr->c.nr.run_off)),
 			  &runlen);
 
 
@@ -829,10 +841,44 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			if (getu16(fs, attr->flags) & NTFS_ATTR_FLAG_SPAR)
 				data_flag |= FS_DATA_SPAR;
 
+			/* SPECIAL CASE 
+			 * We are in non-res section, so we know this
+			 * isn't $STD_INFO and $FNAME
+			 *
+			 * When we are processing a non-base entry, we may
+			 * find an attribute with an id of 0 and it is an
+			 * extention of a previous run (i.e. non-zero start VCN)
+			 * 
+			 * We will lookup if we already have such an attribute
+			 * and get its ID
+			 *
+			 * We coudl also check for a start_vcn if this does
+			 * not fix the problem
+			 */
+			if (id == 0) {
+				FS_DATA *fs_data2 = fs_inode->attr;
+
+				while ((fs_data2) && (fs_data2->flags & FS_DATA_INUSE)) {
+
+					/* We found an attribute with the same name and type */
+					if ((fs_data2->type == type) &&
+					  (strcmp(fs_data2->name, name) == 0)) {
+						id = fs_data2->id;
+						if (verbose)
+							fprintf(logfp, 
+							  "ntfs_proc_attrseq: Updating id from 0 to %d\n", id);
+
+
+						break;
+					}
+					fs_data2 = fs_data2->next;
+				}
+			}
+
 			/* Add the run to the list */
 			fs_inode->attr = fs_data_put_run(fs_inode->attr, 
 			  getu64(fs, attr->c.nr.start_vcn), runlen, fs_data_run,
-			  name, type, getu16(fs, attr->id), 
+			  name, type, id,
 			  getu64(fs, attr->c.nr.ssize), data_flag);
 		}
 
@@ -849,7 +895,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			if (attr->res != NTFS_MFT_RES) 
 				error ("Standard Information Attribute is not resident!");
 
-			si = (ntfs_attr_si *) ((int)attr + getu16(fs, attr->c.r.soff));
+			si = (ntfs_attr_si *) ((uintptr_t)attr + getu16(fs, attr->c.r.soff));
 
 			fs_inode->mtime = nt2unixtime(getu64(fs, si->mtime)); 
 			fs_inode->atime = nt2unixtime(getu64(fs, si->atime)); 
@@ -875,7 +921,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			if (attr->res != NTFS_MFT_RES) 
 				error ("File Name Attribute is not resident!");
 
-			fname = (ntfs_attr_fname *)((int)attr + getu16(fs, attr->c.r.soff));
+			fname = (ntfs_attr_fname *)((uintptr_t)attr + getu16(fs, attr->c.r.soff));
 
 			if (fname->nspace == NTFS_FNAME_DOS)
 				continue;
@@ -953,7 +999,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 
 			for (fs_data = fs_inode->attr; 
 			  fs_data != NULL;
-		  	  fs_data = fs_data->next) {
+			  fs_data = fs_data->next) {
 
 				if ( (fs_data->flags & FS_DATA_INUSE) &&
 			 	  (fs_data->type == NTFS_ATYPE_DATA) &&
@@ -962,6 +1008,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 					break;
 				}
 			}
+			// @@@ Is this needed here -- maybe it should be only in _open
 			if (!ntfs->mft_data) {
 				error ("$Data not found while loading the MFT");
 			}
@@ -1013,7 +1060,7 @@ buffer_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 	len = ((size < act_bufleft) ? size : act_bufleft);
 
 	memcpy(act_buf, buf, len);
-	act_buf = (char *)((int)act_buf + (int)len);
+	act_buf = (char *)((uintptr_t)act_buf + len);
 	act_bufleft -= len;
    
        	/* If any cluster is unallocated, then set a flag */	
@@ -1024,6 +1071,8 @@ buffer_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 }
 
 
+
+
 /*
  * Attribute lists are used when all of the attribute  headers can not
  * fit into one MFT entry.  This contains an entry for every attribute
@@ -1037,7 +1086,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 {
 	ntfs_attrlist *list;
 	char *buf;
-	DADDR_T endaddr;
+	uintptr_t endaddr;
 	FS_INFO *fs = (FS_INFO *)&ntfs->fs_info;
 	ntfs_mft *mft = (ntfs_mft *)mymalloc (ntfs->mft_rsize_b);
 
@@ -1058,7 +1107,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 	/* Get a copy of the attribute list stream using the above action*/
 	act_bufleft = (int)fs_data_attrlist->size;
 	act_buf = buf = mymalloc (act_bufleft);
-	endaddr = (DADDR_T)buf + fs_data_attrlist->size;
+	endaddr = (uintptr_t)buf + fs_data_attrlist->size;
 
 	ntfs_data_walk(ntfs, fs_data_attrlist, 0,
 	  buffer_action, "attrlist");
@@ -1072,8 +1121,8 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 
 	/* Process the list & and call ntfs_proc_attr */
 	for (list = (ntfs_attrlist *)buf;
-	  (list) && ((DADDR_T)list < endaddr);
-	  list = (ntfs_attrlist *)((int)list + getu16(fs, list->len)) )
+	  (list) && ((uintptr_t)list < endaddr);
+	  list = (ntfs_attrlist *)((uintptr_t)list + getu16(fs, list->len)) )
 	{
 		int mftnum;
 		u_int32_t type;
@@ -1104,8 +1153,9 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 		id = getu16(fs, list->id);
 
 		if (verbose)
-			fprintf(logfp, "ntfs_proc_attrlist: mft: %lu type %lu id %lu\n", 
-			  (ULONG)mftnum, (ULONG)type, (ULONG)id);
+			fprintf(logfp, "ntfs_proc_attrlist: mft: %lu type %lu id %lu  VCN: %lu\n", 
+			  (ULONG)mftnum, (ULONG)type, (ULONG)id, (ULONG)
+			  (ULONG)getu64(fs, list->start_vcn));
 
 		/* 
 		 * Read the MFT entry 
@@ -1154,7 +1204,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 		 */
 
 		ntfs_proc_attrseq (ntfs, fs_inode, 
-		  (ntfs_attr *) ((int)mft + (int)getu16(fs, mft->attr_off)),
+		  (ntfs_attr *) ((uintptr_t)mft + getu16(fs, mft->attr_off)),
 	  	  ntfs->mft_rsize_b - getu16(fs, mft->attr_off));
 	}
 
@@ -1230,7 +1280,7 @@ ntfs_copy_inode(NTFS_INFO *ntfs, FS_INODE *fs_inode)
 	/* Process the attribute sequence to fill in the fs_inode->attr
 	 * list and the other info such as size and times
 	 */
-	attr = (ntfs_attr *) ((int)mft + (int)getu16(fs, mft->attr_off));
+	attr = (ntfs_attr *) ((uintptr_t)mft + getu16(fs, mft->attr_off));
 	ntfs_proc_attrseq (ntfs, fs_inode, attr, 
 	  ntfs->mft_rsize_b - getu16(fs, mft->attr_off));
 
@@ -1302,14 +1352,14 @@ ntfs_load_attrdef (NTFS_INFO *ntfs)
 	if (!fs_data)
 		error ("Data attribute not found in $Attr");
  
-    act_bufleft = fs_data->size; 
+	act_bufleft = fs_data->size; 
 	act_buf = mymalloc(act_bufleft);
-    ntfs->attrdef = (ntfs_attrdef *)act_buf;
+	ntfs->attrdef = (ntfs_attrdef *)act_buf;
     
-    ntfs_data_walk(ntfs, fs_data, 0, buffer_action, "");
+	ntfs_data_walk(ntfs, fs_data, 0, buffer_action, "");
 
-    if (act_bufleft)
-        error ("space still left after walking $Attr data");
+	if (act_bufleft)
+		error ("space still left after walking $Attr data");
 
 	fs_inode_free(fs_inode);
 	return;
@@ -1353,16 +1403,16 @@ ntfs_load_bmap(NTFS_INFO *ntfs)
 	/* Get data on the bitmap */
 	ntfs_dinode_lookup (ntfs, NTFS_MFT_BMAP);
 
-	attr = (ntfs_attr *) ( (int)ntfs->mft + 
-	  (int)getu16(fs, ntfs->mft->attr_off));
+	attr = (ntfs_attr *) ( (uintptr_t)ntfs->mft + 
+	  getu16(fs, ntfs->mft->attr_off));
 
 	/* cycle through them */
-	while (((int)attr >= (int)ntfs->mft) &&
-        ((int)attr <= ((int)ntfs->mft + (int)ntfs->mft_rsize_b)) &&
+	while (((uintptr_t)attr >= (uintptr_t)ntfs->mft) &&
+        ((uintptr_t)attr <= ((uintptr_t)ntfs->mft + (uintptr_t)ntfs->mft_rsize_b)) &&
         (getu32(fs, attr->len) > 0 &&
         (getu32(fs, attr->type) != 0xffffffff) &&
         (getu32(fs, attr->type) != NTFS_ATYPE_DATA))) {
-		attr = (ntfs_attr *)((int)attr + getu32(fs, attr->len));
+		attr = (ntfs_attr *)((uintptr_t)attr + getu32(fs, attr->len));
 	}
 
 	/* did we get it? */
@@ -1371,11 +1421,11 @@ ntfs_load_bmap(NTFS_INFO *ntfs)
 
 	/* convert to generic form */
 	ntfs->bmap = ntfs_make_data_run(ntfs, 
-	  (ntfs_runlist *) ((int)attr + getu16(fs, attr->c.nr.run_off)), NULL);
+	  (ntfs_runlist *) ((uintptr_t)attr + getu16(fs, attr->c.nr.run_off)), NULL);
 
 	ntfs->bmap_buf = fs_buf_alloc(fs->block_size);
 
-	/* Load the first cluster so that we ahve something there */
+	/* Load the first cluster so that we have something there */
 	ntfs->bmap_buf_off = 0;
 	fs_read_block(fs, ntfs->bmap_buf, fs->block_size, ntfs->bmap->addr, "bmap");
 
@@ -1771,13 +1821,13 @@ static void
 ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock, 
   int32_t sec_skew)
 {
-    FS_INODE *fs_inode;
+	FS_INODE *fs_inode;
 	FS_DATA *fs_data;
 	NTFS_INFO *ntfs = (NTFS_INFO *)fs;
 
-    fs_inode = ntfs_inode_lookup (fs, inum);
+	fs_inode = ntfs_inode_lookup (fs, inum);
 	fprintf(hFile, "MFT Entry Header Values:\n");
-    fprintf(hFile, "Entry: %lu        Sequence: %lu\n", 
+	fprintf(hFile, "Entry: %lu        Sequence: %lu\n", 
 	   (ULONG) inum, (ULONG)fs_inode->seq);
 
 	if (getu48(fs, ntfs->mft->base_ref) != 0) {
@@ -2093,14 +2143,14 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	if(fs_data) {
 		char *buf;
 		ntfs_attrlist *list;
-		int endaddr;
+		uintptr_t endaddr;
 
 		fprintf(hFile, "\n$ATTRIBUTE_LIST Attribute Values:\n");
 
 		/* Get a copy of the attribute list stream using the above action*/
 		act_bufleft = (int)fs_data->size;
 		act_buf = buf = mymalloc (act_bufleft);
-		endaddr = (int)buf + fs_data->size;
+		endaddr = (uintptr_t)buf + fs_data->size;
 
 		ntfs_data_walk(ntfs, fs_data, 0,
 		  buffer_action, "attrlist");
@@ -2115,8 +2165,8 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 
 		/* Process the list & print the details */
 		for (list = (ntfs_attrlist *)buf;
-		  (list) && ((int)list < endaddr);
-		  list = (ntfs_attrlist *)((int)list + getu16(fs, list->len)) )
+		  (list) && ((uintptr_t)list < endaddr);
+		  list = (ntfs_attrlist *)((uintptr_t)list + getu16(fs, list->len)) )
 		{
 			fprintf(hFile, "Type: %d-%d \tMFT Entry: %"PRIu64" \tVCN: %"PRIu64"\n",
 			  getu32(fs, list->type), getu16(fs, list->id),
@@ -2128,41 +2178,71 @@ egress:
 	}
 
 	/* Print all of the attributes */
-    fs_data = fs_inode->attr;
+	fs_data = fs_inode->attr;
 
-    fprintf(hFile, "\nAttributes: \n");
-    while ((fs_data) && (fs_data->flags & FS_DATA_INUSE)) {
-        char type[256];
+	fprintf(hFile, "\nAttributes: \n");
+	while ((fs_data) && (fs_data->flags & FS_DATA_INUSE)) {
+
+		char type[256];
     
-        ntfs_attrname_lookup(fs, fs_data->type, type, 256);
+		ntfs_attrname_lookup(fs, fs_data->type, type, 256);
 
-        printf("Type: %s (%i-%i)   Name: %s   %sResident%s%s%s   size: %llu\n",
-          type, fs_data->type, fs_data->id, fs_data->name,
-          (fs_data->flags & FS_DATA_NONRES)?"Non-":"", 
-          (fs_data->flags & FS_DATA_ENC)?", Encrypted":"", 
-          (fs_data->flags & FS_DATA_COMP)?", Compressed":"", 
-          (fs_data->flags & FS_DATA_SPAR)?", Sparse":"", 
+		printf("Type: %s (%i-%i)   Name: %s   %sResident%s%s%s   size: %llu\n",
+		  type, fs_data->type, fs_data->id, fs_data->name,
+		  (fs_data->flags & FS_DATA_NONRES)?"Non-":"", 
+		  (fs_data->flags & FS_DATA_ENC)?", Encrypted":"", 
+		  (fs_data->flags & FS_DATA_COMP)?", Compressed":"", 
+		  (fs_data->flags & FS_DATA_SPAR)?", Sparse":"", 
 		  (ULLONG)fs_data->size);
 
    		/* print the layout if it is non-resident and not "special" */
-        if (fs_data->flags & FS_DATA_NONRES) {
-            printidx = 0;
+		if (fs_data->flags & FS_DATA_NONRES) {
+			printidx = 0;
 
-            fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
+			fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
 			   (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT),
-               print_addr_act, (char *)hFile);
-            if (printidx != 0)
-                printf("\n");
-        }
+			  print_addr_act, (char *)hFile);
 
-        fs_data = fs_data->next;
-    }
+			if (printidx != 0)
+				printf("\n");
+		}
+
+		fs_data = fs_data->next;
+	}
 
 	fs_inode_free (fs_inode);
 
     return;
 }
 
+
+
+/* JOURNAL CODE - MOVE TO NEW FILE AT SOME POINT */
+
+void    
+ntfs_jopen (FS_INFO *fs, u_int32_t inum)
+{
+	fprintf(stderr, "NTFS Journal is not yet supported\n");
+	exit(1);
+}
+
+void
+ntfs_jentry_walk (FS_INFO *fs, int flags, FS_JENTRY_WALK_FN action, char *ptr)
+{
+	fprintf(stderr, "NTFS Journal is not yet supported\n");
+	exit(1);
+}
+
+
+void
+ntfs_jblk_walk (FS_INFO *fs, u_int32_t start, u_int32_t end, int flags, FS_JBLK_WALK_FN action, char *ptr)
+{
+	fprintf(stderr, "NTFS Journal is not yet supported\n");
+	exit(1);
+}
+
+
+
 static void
 ntfs_close(FS_INFO *fs)
 {
@@ -2188,6 +2268,8 @@ ntfs_open(const char *name, unsigned char ftype)
 	int len;
 
 	loading_the_MFT = 0;
+	ntfs->bmap = NULL;
+	ntfs->bmap_buf = NULL;
 
 	if ((ftype & FSMASK) != NTFS_TYPE)
 		error ("Invalid FS type in ntfs_open");
@@ -2306,6 +2388,11 @@ ntfs_open(const char *name, unsigned char ftype)
 	ntfs->attrdef = NULL;
 
 
+	fs->jblk_walk = ntfs_jblk_walk;
+	fs->jentry_walk = ntfs_jentry_walk;
+	fs->jopen = ntfs_jopen;
+	fs->journ_inum = 0;
+
 	if (verbose) {
 		fprintf(logfp,
 			"ssize: %d csize: %d serial: %x\n",
