@@ -90,7 +90,7 @@ idxalloc_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 	len = ((size < bufleft) ? size : bufleft);
 
 	memcpy(curbuf, buf, len);
-	curbuf = (char *)((int)curbuf + len);
+	curbuf = (char *)((uintptr_t)curbuf + len);
 	bufleft -= len;
 
 	return ((bufleft > 0) ? WALK_CONT : WALK_STOP);
@@ -134,7 +134,7 @@ static void
 ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
   int size, int len, int flags, FS_DENT_WALK_FN action, char *ptr)
 {
-	DADDR_T endaddr, endaddr_alloc;
+	uintptr_t endaddr, endaddr_alloc;
 	int myflags = 0;
 	FS_DENT *fs_dent = fs_dent_alloc(NTFS_MAXNAMLEN);
 	FS_INFO *fs = (FS_INFO *)&ntfs->fs_info;
@@ -145,13 +145,13 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		  (ULONG)idxe, (ULONG)size, (ULONG)len);
 
 	/* where is the end of the buffer */
-	endaddr = ((int)idxe + size);
+	endaddr = ((uintptr_t)idxe + size);
 
 	/* where is the end of the allocated data */
-	endaddr_alloc = ((int)idxe + len);
+	endaddr_alloc = ((uintptr_t)idxe + len);
 
 	/* cycle through the index entries, based on provided size */
-	while (((int)&(idxe->stream) + sizeof(ntfs_attr_fname)) < endaddr) {
+	while (((uintptr_t)&(idxe->stream) + sizeof(ntfs_attr_fname)) < endaddr) {
 
 		ntfs_attr_fname *fname = (ntfs_attr_fname *)&idxe->stream;
 
@@ -171,21 +171,21 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		   (getu16(fs, idxe->idxlen) % 4) || 
 		   (getu16(fs, idxe->idxlen) > size) )
 		{
-			idxe = (ntfs_idxentry *)((int)idxe + 4);
+			idxe = (ntfs_idxentry *)((uintptr_t)idxe + 4);
 			continue;
 		}
 
 		/* do some sanity checks on the deleted entries
 		 */
 		if  ( (getu16(fs, idxe->strlen) == 0) ||
-		  (((int)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
+		  (((uintptr_t)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
 
 			/* name space checks */
 			if ((fname->nspace != NTFS_FNAME_POSIX) &&
 			  (fname->nspace != NTFS_FNAME_WIN32) &&
 			  (fname->nspace != NTFS_FNAME_DOS) &&
 			  (fname->nspace != NTFS_FNAME_WINDOS) ) {
-				idxe = (ntfs_idxentry *)((int)idxe + 4);
+				idxe = (ntfs_idxentry *)((uintptr_t)idxe + 4);
 				continue;
 			}
 
@@ -194,7 +194,7 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 			  (fname->nlen == 0) ||
 			  (*(u_int8_t *)&fname->name == 0)) {
 
-				idxe = (ntfs_idxentry *)((int)idxe + 4);
+				idxe = (ntfs_idxentry *)((uintptr_t)idxe + 4);
 				continue;
 			}
 
@@ -202,7 +202,7 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 			  (is_time(getu64(fs, fname->atime)) == 0) ||
 			  (is_time(getu64(fs, fname->mtime)) == 0) ) { 
 
-				idxe = (ntfs_idxentry *)((int)idxe + 4);
+				idxe = (ntfs_idxentry *)((uintptr_t)idxe + 4);
 				continue;
 
 			}
@@ -238,7 +238,7 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 			  "ntfs_dent_idxentry: Deletion Check Details of %s: Str Len: %lu  Len to end after current: %lu\n",
 			  fs_dent->name, 
 			  (ULONG)getu16(fs, idxe->strlen), 
-			  (ULONG)(endaddr_alloc - (int)idxe - getu16(fs, idxe->idxlen)));
+			  (ULONG)(endaddr_alloc - (uintptr_t)idxe - getu16(fs, idxe->idxlen)));
 			  
 		/* 
 		 * Check if this entry is deleted
@@ -247,7 +247,7 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		 * within the space that the idxallocbuf claimed was valid
 		 */
 		if ((getu16(fs, idxe->strlen) == 0) || 
-		  (((int)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
+		  (((uintptr_t)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
 
 			/* we know deleted entries with an inode of 0 are not legit because
 			 * that is the MFT value.  Free it so it does not confuse
@@ -264,7 +264,7 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 
 
 		if ((flags & myflags) == myflags) {
-			if (WALK_STOP == action (fs, fs_dent, myflags, "")) { 
+			if (WALK_STOP == action (fs, fs_dent, myflags, ptr)) { 
 				return;
 			}
 		}
@@ -306,9 +306,9 @@ incr_entry:
 		 * 2*nlen: size of name (in unicode)
 		 */
 		if (getu16(fs, idxe->strlen) == 0)
-			idxe = (ntfs_idxentry *)((((int)idxe + 16 + 66 + 2*fname->nlen + 3)/4)*4);
+			idxe = (ntfs_idxentry *)((((uintptr_t)idxe + 16 + 66 + 2*fname->nlen + 3)/4)*4);
 		else
-			idxe = (ntfs_idxentry *)((int)idxe + getu16(fs, idxe->idxlen));
+			idxe = (ntfs_idxentry *)((uintptr_t)idxe + getu16(fs, idxe->idxlen));
 
 	} /* end of loop of index entries */
 
@@ -344,7 +344,7 @@ ntfs_fix_idxrec(NTFS_INFO *ntfs, ntfs_idxrec *idxrec, u_int32_t len)
 		error ("More Update Sequence Entries than idx record size");
 
 	/* Apply the update sequence structure template */
-	upd = (ntfs_upd *)((int)idxrec + getu16(fs, idxrec->upd_off));
+	upd = (ntfs_upd *)((uintptr_t)idxrec + getu16(fs, idxrec->upd_off));
    
 	/* Get the sequence value that each 16-bit value should be */
 	orig_seq = getu16(fs, upd->upd_val);
@@ -357,7 +357,7 @@ ntfs_fix_idxrec(NTFS_INFO *ntfs, ntfs_idxrec *idxrec, u_int32_t len)
 		u_int8_t *new, *old;
 
 		/* get the current sequence value */
-		u_int16_t cur_seq = getu16(fs, (int)idxrec + offset);
+		u_int16_t cur_seq = getu16(fs, (uintptr_t)idxrec + offset);
 
 		if (cur_seq != orig_seq) {
 			/* get the replacement value */
@@ -368,7 +368,7 @@ ntfs_fix_idxrec(NTFS_INFO *ntfs, ntfs_idxrec *idxrec, u_int32_t len)
 		}
 
 		new = &upd->upd_seq + (i-1) * 2;
-		old = (u_int8_t *)(int)idxrec + offset;
+		old = (u_int8_t *)((uintptr_t)idxrec + offset);
 
 		if (verbose)
 			fprintf(logfp,
@@ -450,7 +450,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode);
 		fs_dent->ent_type = FS_DENT_DIR;
 
-		if (WALK_STOP == action (fs, fs_dent, myflags, "")) { 
+		if (WALK_STOP == action (fs, fs_dent, myflags, ptr)) { 
 			fs_dent_free(fs_dent);
 			fs_inode_free(fs_inode);
 			return;
@@ -477,7 +477,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 			
 			fs_dent->inode = fs_name->par_inode;
 			fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode);
-			if (WALK_STOP == action (fs, fs_dent, myflags, "")) { 
+			if (WALK_STOP == action (fs, fs_dent, myflags, ptr)) { 
 				fs_dent_free(fs_dent);
 				fs_inode_free (fs_inode);
 				return;
@@ -512,7 +512,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 	idxelist = &idxroot->list;
 
 	/* Get the offset to the start of the index entry list */
-	idxe = (ntfs_idxentry *)((int)idxelist +
+	idxe = (ntfs_idxentry *)((uintptr_t)idxelist +
 		  getu32(fs, idxelist->begin_off));
 
 	/* Process $IDX_ROOT  */
@@ -610,7 +610,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		/* idxrec points to the next idxrec structure, idxrec_p
 		 * points to the one we are going to process
 		 */
-		rec_len = ((int)idxrec - (int)idxrec_p);
+		rec_len = ((uintptr_t)idxrec - (uintptr_t)idxrec_p);
 
 		if (verbose)
 			fprintf (logfp, 
@@ -622,7 +622,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 
 		/* Locate the start of the index entry list */
 		idxelist = &idxrec_p->list;
-		idxe = (ntfs_idxentry *)((int)idxelist +
+		idxe = (ntfs_idxentry *)((uintptr_t)idxelist +
 		  getu32(fs, idxelist->begin_off));
 
 		/* the length from the start of the next record to where our
@@ -630,7 +630,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		 * This should be the same as buf_off in idxelist, but we don't
 		 * trust it.
 		 */
-		list_len = (int)idxrec - (int)idxe;
+		list_len = (uintptr_t)idxrec - (uintptr_t)idxe;
 
 		/* process the list of index entries */
 		ntfs_dent_idxentry (ntfs, idxe, list_len,
@@ -648,7 +648,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		int list_len, rec_len;
 
 		/* Length from end of attribute to start of this */
-		rec_len = idxalloc_len - ((int)idxrec_p - (int)idxalloc);
+		rec_len = idxalloc_len - ((uintptr_t)idxrec_p - (uintptr_t)idxalloc);
 
 		if (verbose)
 			fprintf (logfp, 
@@ -659,11 +659,11 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		ntfs_fix_idxrec(ntfs, idxrec_p, rec_len);
 
 		idxelist = &idxrec_p->list;
-		idxe = (ntfs_idxentry *)((int)idxelist + 
+		idxe = (ntfs_idxentry *)((uintptr_t)idxelist + 
 		  getu32(fs, idxelist->begin_off));
 
 		/* This is the length of the idx entries */
-		list_len = ((int)idxalloc + idxalloc_len) - (int)idxe;
+		list_len = ((uintptr_t)idxalloc + idxalloc_len) - (uintptr_t)idxe;
 
 		/* process the list of index entries */
 		ntfs_dent_idxentry (ntfs, idxe, list_len, 
@@ -724,10 +724,9 @@ ntfs_find_file_rec (FS_INFO *fs, FS_DENT *fs_dent, FS_NAME *fs_name,
 		/* @@@ There should be a sanity check here to verify that the 
 		 * previous name was unallocated ... but how do I get it again?
 		 */
-
-		if ((((int)didx[depth-1] - len) >= (int)&dirs[0] ) &&
+		if ((((uintptr_t)didx[depth-1] - len) >= (uintptr_t)&dirs[0] ) &&
 		  (depth < MAX_DEPTH)) {
-			begin = didx[depth] = (char *)((int)didx[depth-1] - len);
+			begin = didx[depth] = (char *)((uintptr_t)didx[depth-1] - len);
 
 			depth++;
 			decrem = 1;
@@ -754,9 +753,9 @@ ntfs_find_file_rec (FS_INFO *fs, FS_DENT *fs_dent, FS_NAME *fs_name,
 
 		/* do some length checks on the dir structure 
 		 * if we can't fit it then forget about it */
-		if ((((int)didx[depth-1] - len - 1) >= (int)&dirs[0] ) &&
+		if ((((uintptr_t)didx[depth-1] - len - 1) >= (uintptr_t)&dirs[0] ) &&
 		  (depth < MAX_DEPTH)) {
-			begin = didx[depth] = (char *)((int)didx[depth-1] - len - 1);
+			begin = didx[depth] = (char *)((uintptr_t)didx[depth-1] - len - 1);
 
 			depth++;
 			decrem = 1;
@@ -779,7 +778,7 @@ ntfs_find_file_rec (FS_INFO *fs, FS_DENT *fs_dent, FS_NAME *fs_name,
 			 * if we do then the printed result will have '//' at 
 			 * the beginning
 			 */
-			fs_dent->path = (char *)(int)begin + 1;
+			fs_dent->path = (char *)((uintptr_t)begin + 1);
 			fs_dent->pathdepth = depth;
 			action (fs, fs_dent, myflags, ptr);
 		}
