@@ -33,6 +33,8 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <ctype.h>
+
 #include "mm_tools.h"
 #include "error.h"
 
@@ -88,3 +90,40 @@ guessu32(MM_INFO *mm, u_int8_t *x, u_int32_t val)
 	return 1;
 }
 
+
+/*
+ * 
+ *  * both uni and asc must be defined and asc must be >= ulen / 2
+ *   */
+void
+uni2ascii(u_int8_t *uni, int ulen, u_int8_t *asc, int alen)
+{
+	int i, l;
+
+
+	/* find the maximum that we can go
+	 * we will break when we hit NULLs, but this is the
+	 * absolute max 
+	 */
+	if ((ulen + 1) > alen)
+		l = alen - 1;
+	else
+		l = ulen;
+
+	for (i = 0; i < l; i++) {
+		/* If this value is NULL or 0xff, then stop */
+		if ((uni[i*2] == 0 && uni[i*2 + 1] == 0) ||
+		  (uni[i*2] == 0xff && uni[i*2 + 1] == 0xff))
+			break;
+
+		if (isprint((int)uni[i*2]))
+			asc[i] = uni[i*2];
+		else
+			asc[i] = '?';
+	}
+
+	/* NULL Terminate */
+	asc[i] = '\0';
+	return;
+}
+
