@@ -56,6 +56,8 @@ mm_open(const char *path, const char *type, daddr_t sect_offset)
 		  return bsd_open(path, MM_BSD, sect_offset);
 		case MM_SUN:
 		  return sun_open(path, MM_SUN, sect_offset);
+		case MM_GPT:
+		  return gpt_open(path, MM_GPT, sect_offset); 
 		case MM_UNSUPP:
 		default:
 			printf ("unknown partition type: %s\n", type);
